﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using CustomAvatarAnimation;
using AvatarBlockWar.ParticleEffects;

namespace AvatarBlockWar.AvatarModels
{
    /// <summary>
    /// The possible animation types on the avatar.
    /// </summary>
    public enum AnimationType
    {
        Idle1, Idle2, Idle3, Idle4, Walk,
        Jump, Kick, Punch, Faint, AvatarHoldsWeapon
    };

    /// <summary>
    /// This class represent a avatar model.
    /// </summary>
    public class AvatarModel
    {

        #region Variables

        // The AvatarDescription and AvatarRenderer that are used to render the avatar
        private AvatarRenderer avatarRenderer;
        private AvatarDescription avatarDescription;

        // Animations that will be used
        private IAvatarAnimation[] animations;

        // Tells us if we are using an idle, walking, or other animations
        public AnimationType currentType;

        // Avatar world
        private Matrix avatarWorld;

        // Place a weapon in the hands of the Avatar
        private Model assaultRifle;
        private List<Matrix> bonesWorldSpace;

        // Particle effects
        public EnemyRifleShotEffect shotEffect;
        public BloodEffect bloodEffect;

        // A random number generator for picking new idle animations
        private Random random = new Random();

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new avatar model
        /// </summary>
        public AvatarModel(ContentManager content, GraphicsDevice graphicsDevice)
        {
            // Load avatar content
            LoadContent(content);

            // Create effects
            this.shotEffect = new EnemyRifleShotEffect(content, graphicsDevice);
            this.bloodEffect = new BloodEffect(content, graphicsDevice);

        } // End of the constructor

        #endregion

        #region Initialization

        /// <summary>
        /// Load the graphical content for the avatar.
        /// </summary>
        private void LoadContent(ContentManager content)
        {
            // Load custom animations
            CustomAvatarAnimationData animationData;

            // We will use 9 different animations
            animations = new IAvatarAnimation[10];

            // Load the idle animations
            for (int i = 0; i < 4; i++)
            {
                animations[i] = new AvatarAnimation(
                         (AvatarAnimationPreset)((int)AvatarAnimationPreset.Stand0 + i));
            }

            // Load the walk animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/Walk");
            animations[4] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // Load the jump animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/Jump");
            animations[5] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // Load the kick animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/Kick");
            animations[6] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // Load the punch animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/Punch");
            animations[7] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // Load the faint animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/Faint");
            animations[8] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // Load the hold weapon animation
            animationData = content.Load<CustomAvatarAnimationData>("CustomAvatarAnimations/AvatarHoldsWeapon");
            animations[9] = new CustomAvatarAnimationPlayer(animationData.Name, animationData.Length,
                                          animationData.Keyframes, animationData.ExpressionKeyframes);

            // ******************************************************************************
            // Place assault rifle in the hands of the Avatar
            // ******************************************************************************
            this.assaultRifle = content.Load<Model>("Tools/AssaultRifle/weapon");
            bonesWorldSpace = new List<Matrix>(AvatarRenderer.BoneCount);
            for (int i = 0; i < AvatarRenderer.BoneCount; i++)
                bonesWorldSpace.Add(Matrix.Identity);

            // Select a random idle animation to start
            SetRandomIdleAnimation();

            this.avatarWorld = Matrix.Identity;

        } // End of the LoadContent method

        #endregion

        #region Updating

        /// <summary>
        /// Update the avatar
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="playerPosition"></param>
        /// <param name="avatarRotation"></param>
        public void UpdateAvatar(GameTime gameTime, Vector3 playerPosition, Quaternion avatarRotation)
        {
            // Update the avatar world
            this.avatarWorld = Matrix.CreateScale(40f) * Matrix.Transform(Matrix.Identity, avatarRotation) * Matrix.CreateTranslation(playerPosition);

            // Loop animation if we are walking or holding a weapon
            bool loopAnimation = (currentType == AnimationType.Walk || this.currentType == AnimationType.AvatarHoldsWeapon);

            // Update the current animation and world space bones
            if (avatarRenderer != null && avatarRenderer.State == AvatarRendererState.Ready)
            {
                animations[(int)currentType].Update(gameTime.ElapsedGameTime, loopAnimation);
                BonesToWorldSpace(avatarRenderer, animations[(int)currentType], bonesWorldSpace);
            }

            //// Check to see if we need to end the animation
            //if (!loopAnimation)
            //{
            //    if (animations[(int)currentType].CurrentPosition == animations[(int)currentType].Length)
            //    {
            //        // Start new idle animation
            //        SetRandomIdleAnimation();
            //    }
            //}

        } // End of the UpdateAvatar method.

        #endregion

        #region Drawing

        /// <summary>
        /// Draw the avatar
        /// </summary>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public void DrawAvatar(Matrix view, Matrix projection)
        {
            // Draw the weapon 
            DrawWeapon(view, projection);

            // Draw the avatar
            if (avatarRenderer != null)
            {
                // Set the world, view and projection for the avatar renderer
                avatarRenderer.World = this.avatarWorld;
                avatarRenderer.View = view;
                avatarRenderer.Projection = projection;

                // Draw the avatar
                avatarRenderer.Draw(animations[(int)currentType]);

                // If we are on a pc draw something, like a barrel

            }

        } // End of the DrawAvatar method

        /// <summary>
        /// Helper method to draw the current weapon
        /// </summary>
        private void DrawWeapon(Matrix view, Matrix projection)
        {
            // Moves the bat closer to where we want it in the hand
            Matrix weaponOffset = Matrix.CreateScale(0.01f) * Matrix.CreateRotationX(MathHelper.ToRadians(90)) * Matrix.CreateRotationZ(MathHelper.ToRadians(20));
            weaponOffset.Translation += new Vector3(-0.05f, 0.05f, 0.07f);

            foreach (ModelMesh mesh in this.assaultRifle.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();

                    // Position the bat to be near the avatars right hand. The position
                    // of the right special bone can be found by looking up the value in
                    // our list of world space bones with the index of the bone we are 
                    // looking for. The bat is translated and rotated a small amount to
                    // make it look better in the hand.
                    effect.World = weaponOffset * bonesWorldSpace[(int)AvatarBone.SpecialRight];

                    effect.View = view;
                    effect.Projection = projection;
                }

                mesh.Draw();
            }

        } // End of the DrawWeapon method

        /// <summary>
        /// Updates a list of matrices to represent the location of the 
        /// avatar bones in world space with the avatar animation applied.
        /// </summary>
        private static void BonesToWorldSpace(AvatarRenderer renderer, IAvatarAnimation animation,
                                                        List<Matrix> boneToUpdate)
        {
            // Bind pose of the avatar. 
            // These positions are in local space, and are relative to the parent bone.
            IList<Matrix> bindPose = renderer.BindPose;
            // The current animation pose. 
            // These positions are in local space, and are relative to the parent bone.
            IList<Matrix> animationPose = animation.BoneTransforms;
            // List of parent bones for each bone in the hierarchy 
            IList<int> parentIndex = renderer.ParentBones;

            // Loop all of the bones.
            // Since the bone hierarchy is sorted by depth 
            // we will transform the parent before any child.
            for (int i = 0; i < AvatarRenderer.BoneCount; i++)
            {
                // Find the transform of this bones parent.
                // If this is the first bone use the world matrix used on the avatar
                Matrix parentMatrix = (parentIndex[i] != -1)
                                       ? boneToUpdate[parentIndex[i]]
                                       : renderer.World;
                // Calculate this bones world space position
                boneToUpdate[i] = Matrix.Multiply(Matrix.Multiply(animationPose[i],
                                                                  bindPose[i]),
                                                                  parentMatrix);
            }

        } // End of the BonesToWorldSpace method

        #endregion

        #region Animation Selection

        /// <summary>
        /// Start playing a random idle animation
        /// </summary>
        public void SetRandomIdleAnimation()
        {
            SetAnimation((byte)random.Next(4));
        }

        /// <summary>
        /// Start playing one of the other animations that were loaded
        /// </summary>
        public void SetAnimation(byte animation)
        {
            // Set a new animation if it is different from the current type
            if (animation != (byte)currentType)
            {
                animations[(int)animation].CurrentPosition = TimeSpan.Zero;
                AnimationType animationType = (AnimationType)animation;
                currentType = animationType;
            }
        }

        #endregion

        #region Avatar Loading

        /// <summary>
        /// Load the avatar for a gamer
        /// </summary>
        public void LoadAvatar(Gamer gamer)
        {
            UnloadAvatar();

            AvatarDescription.BeginGetFromGamer(gamer, LoadAvatarDescription, null);
        }

        /// <summary>
        /// AsyncCallback for loading the AvatarDescription
        /// </summary>
        private void LoadAvatarDescription(IAsyncResult result)
        {
            // Get the AvatarDescription for the gamer
            avatarDescription = AvatarDescription.EndGetFromGamer(result);

            // Load the AvatarRenderer if description is valid
            if (avatarDescription.IsValid)
            {
                avatarRenderer = new AvatarRenderer(avatarDescription);
                //avatarRenderer.Projection = projection;
            }
            // Load random for an invalid description
            else
            {
                LoadRandomAvatar();
            }
        }

        /// <summary>
        /// Load a random avatar
        /// </summary>
        public void LoadRandomAvatar()
        {
            UnloadAvatar();

            avatarDescription = AvatarDescription.CreateRandom();
            avatarRenderer = new AvatarRenderer(avatarDescription);
            //avatarRenderer.Projection = projection;
        }

        /// <summary>
        /// Unloads the current avatar
        /// </summary>
        private void UnloadAvatar()
        {
            // Dispose the current Avatar
            if (avatarRenderer != null)
            {
                avatarRenderer.Dispose();
                avatarRenderer = null;
            }
        }

        #endregion

    } // End of the class

} // End of the namespace
