﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using AvatarBlockWar.PlayerProfiles;
using MapObjects;

namespace CubeEngines
{
    /// <summary>
    /// This main class represent a cube engine
    /// </summary>
    public abstract class CubeEngine : IDisposable
    {
        #region Variables

        // A list of vertices and a list of indices that is used to create a cube (24 vertices and 36 indices).
        public List<VertexPositionNormalTexture> cubeVertices = new List<VertexPositionNormalTexture>(24);
        public List<ushort> cubeIndices = new List<ushort>(36);

        // A vertex buffer and a index buffer for a cube.
        public VertexBuffer cubeVertexBuffer;
        public IndexBuffer cubeIndexBuffer;

        // A cube texture map
        public Texture2D cubeTextureMap;
        
        // A reference variable to a list of cube types.
        public CubeType[] cubeTypes;

        // Constant for the mapsize in the direction from the player
        public const short MAP_WIDTH_DEPTH = 16;
        public const short MAP_HEIGHT = 5;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new cube engine.
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="cubeTypes"></param>
        public CubeEngine(GraphicsDevice graphicsDevice)
        {
            // Create the cube
            CreateCube(graphicsDevice, new Vector3(0, 0, 0), new Vector3(40, 40, 40), 1f);

            // Create the cube types list
            this.cubeTypes = new CubeType[36];
            this.cubeTypes[0] = new CubeType("Start block", new Vector2(0, 0), false);
            this.cubeTypes[1] = new CubeType("End block", new Vector2(0.09375f, 0), false);
            this.cubeTypes[2] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[3] = new CubeType("Dark bricks", new Vector2(0.1875f, 0), false);
            this.cubeTypes[4] = new CubeType("Bathroom window", new Vector2(0.28125f, 0), false);
            this.cubeTypes[5] = new CubeType("Window frame", new Vector2(0.375f, 0), true);
            this.cubeTypes[6] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[7] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[8] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[9] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[10] = new CubeType("Template", new Vector2(0.1875f, 0), false);
            this.cubeTypes[11] = new CubeType("Template", new Vector2(0.1875f, 0), false);
            this.cubeTypes[12] = new CubeType("Template", new Vector2(0.1875f, 0), false);
            this.cubeTypes[13] = new CubeType("Template", new Vector2(0.1875f, 0), false);
            this.cubeTypes[14] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[15] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[16] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[17] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[18] = new CubeType("Dark bricks", new Vector2(0.1875f, 0), false);
            this.cubeTypes[19] = new CubeType("Dark bricks", new Vector2(0.1875f, 0), false);
            this.cubeTypes[20] = new CubeType("Dark bricks", new Vector2(0.1875f, 0), false);
            this.cubeTypes[21] = new CubeType("Dark bricks", new Vector2(0.1875f, 0), false);
            this.cubeTypes[22] = new CubeType("Bathroom window", new Vector2(0.28125f, 0), false);
            this.cubeTypes[23] = new CubeType("Bathroom window", new Vector2(0.28125f, 0), false);
            this.cubeTypes[24] = new CubeType("Bathroom window", new Vector2(0.28125f, 0), false);
            this.cubeTypes[25] = new CubeType("Bathroom window", new Vector2(0.28125f, 0), false);
            this.cubeTypes[26] = new CubeType("Window frame", new Vector2(0.375f, 0), true);
            this.cubeTypes[27] = new CubeType("Window frame", new Vector2(0.375f, 0), true);
            this.cubeTypes[28] = new CubeType("Window frame", new Vector2(0.375f, 0), true);
            this.cubeTypes[29] = new CubeType("Window frame", new Vector2(0.375f, 0), true);
            this.cubeTypes[30] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[31] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[32] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[33] = new CubeType("Direction", new Vector2(0, 0), false);
            this.cubeTypes[34] = new CubeType("Crate", new Vector2(0.09375f, 0), false);
            this.cubeTypes[35] = new CubeType("Crate", new Vector2(0.09375f, 0), false);

        } // End of the constructor

        #endregion

        #region Initialization

        /// <summary>
        /// This method is used to create a cube.
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="minPoint"></param>
        /// <param name="maxPoint"></param>
        /// <param name="texCoordScale"></param>
        private void CreateCube(GraphicsDevice graphicsDevice, Vector3 minPoint, Vector3 maxPoint, float texCoordScale)
        {
            // Variables
            Vector3 pointOne;
            Vector3 pointTwo;
            Vector3 pointThree;
            Vector3 pointFour;

            //************************************************************************************************************
            // FRONT FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(minPoint.X, minPoint.Y, maxPoint.Z);// 0
            pointTwo = new Vector3(minPoint.X, maxPoint.Y, maxPoint.Z); // 1
            pointThree = new Vector3(maxPoint.X, maxPoint.Y, maxPoint.Z); // 2
            pointFour = new Vector3(maxPoint.X, minPoint.Y, maxPoint.Z); // 3

            // First triangle
            AddIndex(0);
            AddIndex(1);
            AddIndex(2);

            // Second triangle
            AddIndex(0);
            AddIndex(2);
            AddIndex(3);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0.03125f, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0.03125f, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.0625f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.0625f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END FRONT FACE
            //************************************************************************************************************

            //************************************************************************************************************
            // BACK FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(maxPoint.X, minPoint.Y, minPoint.Z); // 4
            pointTwo = new Vector3(maxPoint.X, maxPoint.Y, minPoint.Z); // 5
            pointThree = new Vector3(minPoint.X, maxPoint.Y, minPoint.Z); // 6
            pointFour = new Vector3(minPoint.X, minPoint.Y, minPoint.Z);// 7

            // First triangle
            AddIndex(4);
            AddIndex(5);
            AddIndex(6);

            // Second triangle
            AddIndex(4);
            AddIndex(6);
            AddIndex(7);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0.03125f, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0.03125f, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.0625f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.0625f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END BACK FACE
            //************************************************************************************************************

            //************************************************************************************************************
            // RIGHT FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(maxPoint.X, minPoint.Y, maxPoint.Z); // 8
            pointTwo = new Vector3(maxPoint.X, maxPoint.Y, maxPoint.Z); // 9
            pointThree = new Vector3(maxPoint.X, maxPoint.Y, minPoint.Z); // 10
            pointFour = new Vector3(maxPoint.X, minPoint.Y, minPoint.Z); // 11

            // First triangle
            AddIndex(8);
            AddIndex(9);
            AddIndex(10);

            // Second triangle
            AddIndex(8);
            AddIndex(10);
            AddIndex(11);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0.03125f, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0.03125f, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.0625f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.0625f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END RIGHT FACE (Clockwise)
            //************************************************************************************************************

            //************************************************************************************************************
            // LEFT FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(minPoint.X, minPoint.Y, minPoint.Z); // 12
            pointTwo = new Vector3(minPoint.X, maxPoint.Y, minPoint.Z); // 13
            pointThree = new Vector3(minPoint.X, maxPoint.Y, maxPoint.Z); // 14
            pointFour = new Vector3(minPoint.X, minPoint.Y, maxPoint.Z); // 15

            // First triangle
            AddIndex(12);
            AddIndex(13);
            AddIndex(14);

            // Second triangle
            AddIndex(12);
            AddIndex(14);
            AddIndex(15);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0.03125f, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0.03125f, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.0625f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.0625f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END LEFT FACE (Clockwise)
            //************************************************************************************************************

            //************************************************************************************************************
            // TOP FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(minPoint.X, maxPoint.Y, maxPoint.Z); // 16
            pointTwo = new Vector3(minPoint.X, maxPoint.Y, minPoint.Z); // 17
            pointThree = new Vector3(maxPoint.X, maxPoint.Y, minPoint.Z); // 18
            pointFour = new Vector3(maxPoint.X, maxPoint.Y, maxPoint.Z); // 19

            // First triangle
            AddIndex(16);
            AddIndex(17);
            AddIndex(18);

            // Second triangle
            AddIndex(16);
            AddIndex(18);
            AddIndex(19);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.03125f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.03125f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END TOP FACE (Clockwise)
            //************************************************************************************************************

            //************************************************************************************************************
            // BOTTOM FACE (Clockwise)
            //************************************************************************************************************
            pointOne = new Vector3(minPoint.X, minPoint.Y, minPoint.Z); // 20
            pointTwo = new Vector3(minPoint.X, minPoint.Y, maxPoint.Z); // 21
            pointThree = new Vector3(maxPoint.X, minPoint.Y, maxPoint.Z); // 22
            pointFour = new Vector3(maxPoint.X, minPoint.Y, minPoint.Z); // 23

            // First triangle
            AddIndex(20);
            AddIndex(21);
            AddIndex(22);

            // Second triangle
            AddIndex(20);
            AddIndex(22);
            AddIndex(23);

            // Four vertices per face.
            AddVertex(pointOne, new Vector3(0, 1, 0), new Vector2(0.0625f, 0.03125f) * texCoordScale);
            AddVertex(pointTwo, new Vector3(0, 1, 0), new Vector2(0.0625f, 0) * texCoordScale);
            AddVertex(pointThree, new Vector3(0, 1, 0), new Vector2(0.09375f, 0) * texCoordScale);
            AddVertex(pointFour, new Vector3(0, 1, 0), new Vector2(0.09375f, 0.03125f) * texCoordScale);
            //************************************************************************************************************
            // END BOTTOM FACE
            //************************************************************************************************************

            // Initialize the cube by creating a vertex buffer and a index buffer for the cube.
            InitializeCube(graphicsDevice);

        } // End of the CreateCube method

        /// <summary>
        /// Adds a new vertex to the cube. This should only be called during the initialization process.
        /// </summary>
        private void AddVertex(Vector3 position, Vector3 normal, Vector2 textureCoordinates)
        {
            cubeVertices.Add(new VertexPositionNormalTexture(position, normal, textureCoordinates));

        } // End of the AddVertex method

        /// <summary>
        /// Adds a new index to the cube. This should only be called during the initialization process.
        /// </summary>
        private void AddIndex(int index)
        {
            if (index > ushort.MaxValue)
                throw new ArgumentOutOfRangeException("index");

            cubeIndices.Add((ushort)index);

        } // End of the AddIndex method

        /// <summary>
        /// Once all the geometry has been specified by calling AddVertex and AddIndex, this method copies the vertex and index data into 
        /// GPU format buffers, ready for efficient rendering.
        /// </summary>
        protected void InitializeCube(GraphicsDevice graphicsDevice)
        {
            // Create a vertex buffer for a cube, and copy our vertex data into it.
            this.cubeVertexBuffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionNormalTexture), cubeVertices.Count, BufferUsage.None);
            this.cubeVertexBuffer.SetData(cubeVertices.ToArray());

            // Create an index buffer for a cube, and copy our index data into it.
            this.cubeIndexBuffer = new IndexBuffer(graphicsDevice, typeof(ushort), cubeIndices.Count, BufferUsage.None);
            this.cubeIndexBuffer.SetData(cubeIndices.ToArray());

        } // End of the InitializeCube method

        #endregion

        #region Update

        public abstract void UpdateCubeInstances(BlockMap3D blockMap3D, HumanPlayerProfile player);

        #endregion

        #region Draw

        /// <summary>
        /// A abstract method to draw solid cubes.
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public abstract void DrawSolidCubes(GraphicsDevice graphicsDevice, Matrix view, Matrix projection);

        /// <summary>
        /// A abstract method to draw transparent cubes.
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public abstract void DrawTransparentCubes(GraphicsDevice graphicsDevice, Matrix view, Matrix projection);

        #endregion

        #region Dispose
        
        /// <summary>
        /// Finalizer.
        /// </summary>
        ~CubeEngine()
        {
            Dispose(false);
        }

        /// <summary>
        /// Frees resources used by this object.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);

        } // End of the Dispose method.

        /// <summary>
        /// Frees resources used by this object.
        /// </summary>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                // Dispose the geometry vertex buffer
                if (this.cubeVertexBuffer != null)
                    this.cubeVertexBuffer.Dispose();

                // Dispose the cube index buffer
                if (this.cubeIndexBuffer != null)
                    cubeIndexBuffer.Dispose();

            }

        } // End of the Dispose method.

        #endregion

    } // End of the class

} // End of the namespace
