﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using AvatarBlockWar.PlayerProfiles;
using MapObjects;

namespace CubeEngines
{
    /// <summary>
    /// This class is used to handle instanced cubes.
    /// </summary>
    public class InstancedCubeEngine : CubeEngine
    {

        #region Variables

        // Dynamic vertex buffers that includes all cube instances.
        DynamicVertexBuffer solidInstanceVertexBuffer;
        DynamicVertexBuffer transparentInstanceVertexBuffer;

        // Cube effect file
        Effect cubeEffect;

        // A list of cube instances
        CubeInstance[] solidCubeInstances;
        CubeInstance[] transparentCubeInstances;

        // Counters for cubes
        int solidCubeCounter = 0;
        int transparentCubeCounter = 0;
        
        // To store instance transform matrices in a vertex buffer, we use this custom
        // vertex type which encodes 4x4 matrices as a set of four Vector4 values.
        static VertexDeclaration instanceVertexDeclaration = new VertexDeclaration
        (
            new VertexElement(0, VertexElementFormat.Vector4, VertexElementUsage.BlendWeight, 0),
            new VertexElement(16, VertexElementFormat.Vector4, VertexElementUsage.BlendWeight, 1),
            new VertexElement(32, VertexElementFormat.Vector4, VertexElementUsage.BlendWeight, 2),
            new VertexElement(48, VertexElementFormat.Vector4, VertexElementUsage.BlendWeight, 3),
            new VertexElement(64, VertexElementFormat.Vector2, VertexElementUsage.TextureCoordinate, 1) 
        );

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new instanced cube engine
        /// </summary>
        /// <param name="graphicsDevice"></param>
        public InstancedCubeEngine(GraphicsDevice graphicsDevice, ContentManager content)
            : base(graphicsDevice)
        {
            // Load the cube effect
            this.cubeEffect = content.Load<Effect>("Effects/cubeEffect");

            // Load the cube texture map
            this.cubeTextureMap = content.Load<Texture2D>("CubeTextures/cubeTextureMap");

            // Set the texture to use in the cube effect (HLSL)
            this.cubeEffect.Parameters["cubeTextureMap"].SetValue(this.cubeTextureMap);

        } // End of the constructor

        #endregion

        #region Update

        /// <summary>
        /// Update cube instances and aiming collision distance
        /// </summary>
        /// <param name="blockMap3D"></param>
        /// <param name="HumanPlayerProfile"></param>
        public override void UpdateCubeInstances(BlockMap3D blockMap3D, HumanPlayerProfile player)
        {
            // Create a new list of cube instances (Maximal size: 32*32*14)
            this.solidCubeInstances = new CubeInstance[14336];
            this.transparentCubeInstances = new CubeInstance[14336];
            
            // Reset counters
            this.solidCubeCounter = 0;
            this.transparentCubeCounter = 0;

            // Reset collision distances
            player.bulletCollisionDistanceCube = float.MaxValue;
            player.aimingCollisionDistanceCube = float.MaxValue;

            // Create a direction ray for aiming collision
            Vector3 direction = player.playerDirection + player.lookUpDown;
            Ray directionRay = new Ray(player.camera.cameraPosition, direction);

            // Create a bounding frustum
            BoundingFrustum boundingFrustum = new BoundingFrustum(player.camera.view * player.camera.projection);

            // Set start and end tiles.
            int zCounter = 1;
            int xCounter = 1;
            int startX = ((int)player.camera.cameraPosition.X / blockMap3D.cubeSize) - MAP_WIDTH_DEPTH;
            int startZ = ((int)player.camera.cameraPosition.Z / blockMap3D.cubeSize) - MAP_WIDTH_DEPTH;
            int startY = ((int)player.camera.cameraPosition.Y / blockMap3D.cubeSize) + MAP_HEIGHT;
            int endX = ((int)player.camera.cameraPosition.X / blockMap3D.cubeSize) + MAP_WIDTH_DEPTH;
            int endZ = ((int)player.camera.cameraPosition.Z / blockMap3D.cubeSize) + MAP_WIDTH_DEPTH;
            int endY = ((int)player.camera.cameraPosition.Y / blockMap3D.cubeSize) - MAP_HEIGHT;

            if (startX < 0)
                startX = 0;
            if (startZ < 0)
                startZ = 0;
            if (startY >= blockMap3D.mapHeightY)
                startY = blockMap3D.mapHeightY - 1;
            if (endX > blockMap3D.mapWidthX)
                endX = blockMap3D.mapWidthX;
            if (endZ > blockMap3D.mapDepthZ)
                endZ = blockMap3D.mapDepthZ;
            if (endY < 0)
                endY = 0;

            // Check the Z-direction and change start and end tiles accordingly
            if (player.playerDirection.Z < 0)
            {
                startZ = ((int)player.camera.cameraPosition.Z / blockMap3D.cubeSize) + MAP_WIDTH_DEPTH;
                endZ = ((int)player.camera.cameraPosition.Z / blockMap3D.cubeSize) - MAP_WIDTH_DEPTH;
                zCounter = -1;

                if (startZ >= blockMap3D.mapDepthZ)
                    startZ = blockMap3D.mapDepthZ - 1;
                if (endZ < 0)
                    endZ = 0;
            }

            // Check the X-direction and change start and end tiles accordingly
            if (player.playerDirection.X < 0)
            {
                startX = ((int)player.camera.cameraPosition.X / blockMap3D.cubeSize) + MAP_WIDTH_DEPTH;
                endX = ((int)player.camera.cameraPosition.X / blockMap3D.cubeSize) - MAP_WIDTH_DEPTH;
                xCounter = -1;

                if (startX >= blockMap3D.mapWidthX)
                    startX = blockMap3D.mapWidthX - 1;
                if (endX < 0)
                    endX = 0;

            }

            // Create cube instances
            for (int y = startY; y >= endY; y--)
            {
                for (int z = startZ; zCounter < 0 ? z >= endZ : z < endZ; z += zCounter)
                {
                    for (int x = startX; xCounter < 0 ? x >= endX : x < endX; x += xCounter)
                    {
                        // Get the cube type
                        short cubeType = blockMap3D.cubes[x, y, z];

                        // Continue if there is no cube
                        if (cubeType == -1)
                            continue;
                        
                        Vector3 cubePosition = new Vector3(x * blockMap3D.cubeSize, y * blockMap3D.cubeSize, z * blockMap3D.cubeSize);

                        // Discard cubes that are outside the bounding frustum
                        BoundingBox boundingBox = new BoundingBox(cubePosition, new Vector3(cubePosition.X + blockMap3D.cubeSize, cubePosition.Y + blockMap3D.cubeSize, cubePosition.Z + blockMap3D.cubeSize));
                        if (boundingFrustum.Contains(boundingBox) == ContainmentType.Disjoint)
                            continue;
 
                        // Add the cube instance to the list.
                        if (cubeTypes[cubeType].isTransparent == true)
                        {
                            transparentCubeInstances[this.transparentCubeCounter] = new CubeInstance(Matrix.CreateTranslation(cubePosition), this.cubeTypes[cubeType].cubeTextureOffset);
                            this.transparentCubeCounter++;
                        }
                        else
                        {
                            solidCubeInstances[this.solidCubeCounter] = new CubeInstance(Matrix.CreateTranslation(cubePosition), this.cubeTypes[cubeType].cubeTextureOffset);
                            this.solidCubeCounter++;
                        }

                        // Check for a collision with the directionRay for a block addition
                        if (boundingBox.Intersects(directionRay) != null)
                        {
                            // Check for a collision
                            float? minDistance = boundingBox.Intersects(directionRay);

                            // A collision has happened.
                            if (minDistance < player.aimingCollisionDistanceCube)
                            {
                                player.aimingCollisionDistanceCube = (float)minDistance;
                                player.cubeSelectionMenu.selectedBlock = new Int32[] { x, y, z };
                            }
                        }

                        // Check for a collision with a bullet.
                        if (player.bullet != null && boundingBox.Intersects(player.bullet.bulletRay) != null)
                        {
                            // Check for a collision
                            float? minDistance = boundingBox.Intersects(player.bullet.bulletRay);

                            // A collision has happened.
                            if (minDistance < player.bulletCollisionDistanceCube)
                            {
                                player.bulletCollisionDistanceCube = (float)minDistance;
                            }
                        }

                    } // End of X-loop

                } // End of Z-loop

            } // End of Y-loop

        } // End of the UpdateCubeInstances method

        #endregion

        #region Draw

        /// <summary>
        /// Draw solid cubes
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public override void DrawSolidCubes(GraphicsDevice graphicsDevice, Matrix view, Matrix projection)
        {
            // Return if there isn´t any solid cubes.
            if (this.solidCubeCounter == 0)
                return;

            // If we have more instances than room in our dynamic vertex buffer, grow it to the neccessary size.
            if ((this.solidInstanceVertexBuffer == null) || (this.solidCubeCounter > this.solidInstanceVertexBuffer.VertexCount))
            {
                if (this.solidInstanceVertexBuffer != null)
                    this.solidInstanceVertexBuffer.Dispose();

                // Create the dynamic vertex buffer
                if (this.solidCubeCounter > 0)
                    this.solidInstanceVertexBuffer = new DynamicVertexBuffer(graphicsDevice, instanceVertexDeclaration, this.solidCubeCounter, BufferUsage.WriteOnly);
            }

            // Transfer the latest instance transform matrices into the instanceVertexBuffer.
            if (this.solidCubeCounter > 0)
                this.solidInstanceVertexBuffer.SetData(this.solidCubeInstances, 0, this.solidCubeCounter);

            // Set graphic states before the draw method.
            graphicsDevice.BlendState = BlendState.Opaque;        
            graphicsDevice.DepthStencilState = DepthStencilState.Default;
            graphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;

            // Tell the GPU to read from both the model vertex buffer plus our instanceVertexBuffer.
            graphicsDevice.SetVertexBuffers(cubeVertexBuffer, new VertexBufferBinding(this.solidInstanceVertexBuffer, 0, 1));

            // Set the index buffer
            graphicsDevice.Indices = cubeIndexBuffer;

            // Set parameters in the effect file (HLSL)
            this.cubeEffect.CurrentTechnique = this.cubeEffect.Techniques["HardwareInstancing"];
            this.cubeEffect.Parameters["ViewProjection"].SetValue(view * projection);
            
            // Apply the current technique.
            this.cubeEffect.CurrentTechnique.Passes[0].Apply();
            int primitiveCount = cubeIndices.Count / 3;

            // Draw solid cubes
            graphicsDevice.DrawInstancedPrimitives(PrimitiveType.TriangleList, 0, 0, cubeVertices.Count, 0, primitiveCount, this.solidCubeCounter);

        } // End of the DrawSolidCubes method

        /// <summary>
        /// Draw transparent cubes with alpha blending
        /// </summary>
        /// <param name="graphicsDevice"></param>
        /// <param name="view"></param>
        /// <param name="projection"></param>
        public override void DrawTransparentCubes(GraphicsDevice graphicsDevice, Matrix view, Matrix projection)
        {

            // Return if there isn´t any transparent cubes
            if (this.transparentCubeCounter == 0)
                return;

            // Reverse the order of the cubes.
            Array.Reverse(this.transparentCubeInstances, 0, this.transparentCubeCounter);

            // If we have more instances than room in our dynamic vertex buffer, grow it to the neccessary size.
            if ((this.transparentInstanceVertexBuffer == null) || (this.transparentCubeCounter > this.transparentInstanceVertexBuffer.VertexCount))
            {
                if (this.transparentInstanceVertexBuffer != null)
                    this.transparentInstanceVertexBuffer.Dispose();

                // Create the dynamic vertex buffer
                if (this.transparentCubeCounter > 0)
                    this.transparentInstanceVertexBuffer = new DynamicVertexBuffer(graphicsDevice, instanceVertexDeclaration, this.transparentCubeCounter, BufferUsage.WriteOnly);
            }

            // Transfer the latest instance transform matrices into the instanceVertexBuffer.
            if (this.transparentCubeCounter > 0)
                this.transparentInstanceVertexBuffer.SetData(this.transparentCubeInstances, 0, this.transparentCubeCounter);

            // Set graphic states before the draw method.
            graphicsDevice.BlendState = BlendState.NonPremultiplied;
            graphicsDevice.DepthStencilState = DepthStencilState.Default;
            graphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;

            // Tell the GPU to read from both the model vertex buffer plus our instanceVertexBuffer.
            graphicsDevice.SetVertexBuffers(cubeVertexBuffer, new VertexBufferBinding(this.transparentInstanceVertexBuffer, 0, 1));

            // Set the index buffer
            graphicsDevice.Indices = cubeIndexBuffer;

            // Set parameters in the effect file (HLSL)
            this.cubeEffect.CurrentTechnique = this.cubeEffect.Techniques["HardwareInstancing"];
            this.cubeEffect.Parameters["ViewProjection"].SetValue(view * projection);

            // Apply the current technique.
            this.cubeEffect.CurrentTechnique.Passes[0].Apply();
            int primitiveCount = cubeIndices.Count / 3;

            // Draw solid cubes
            graphicsDevice.DrawInstancedPrimitives(PrimitiveType.TriangleList, 0, 0, cubeVertices.Count, 0, primitiveCount, this.transparentCubeCounter);

        } // End of the DrawTransparentCubes method.

        #endregion

        #region Dispose
        
        /// <summary>
        /// Frees resources used by this object.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            // Call the dispose method in the base class
            base.Dispose(disposing);

            if (disposing)
            {
                // Dispose the dynamic vertex buffer
                if (this.solidInstanceVertexBuffer != null)
                    this.solidInstanceVertexBuffer.Dispose();

                // Dispose the dynamic vertex buffer
                if (this.transparentInstanceVertexBuffer != null)
                    this.transparentInstanceVertexBuffer.Dispose();
            }

        } // End of the Dispose method.

        #endregion

    } // End of the class

} // End of the namespace
