﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.IsolatedStorage;
using System.Xml;
using System.Xml.Serialization;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.Screens.NetworkScreens;
using ScreenManagement;
using MapObjects;

namespace AvatarBlockWar.DataStorage
{
    /// <summary>
    /// This static class is used to save and open maps
    /// </summary>
    public static class MapDataStorage
    {
        
        #region Variables

        const string displayNameContainer = "AvatarBlockWar-Maps";
        public static BlockMap3D staticBlockMap3D;
        public static string staticMapName;
        public static StorageDevice staticStorageDevice;
        public static ScreenManager screenManager;

        #endregion

        #region Save methods

        /// <summary>
        /// Begin to save a map to a file.
        /// </summary>
        /// <param name="data"></param>
        /// <param name="mapName"></param>
        /// <param name="screenManager"></param>
        /// <param name="storageDevice"></param>
        public static void BeginSaveMapToFile(BlockMap3D data, string mapName, ScreenManager screenManager, StorageDevice storageDevice)
        {
            // Set class variables
            MapDataStorage.staticBlockMap3D = data;
            MapDataStorage.staticMapName = mapName + ".map";
            MapDataStorage.staticStorageDevice = storageDevice;
            MapDataStorage.screenManager = screenManager;

            try
            {

                // Begin to open the container
                storageDevice.BeginOpenContainer(displayNameContainer, new AsyncCallback(SaveMapToFile), null);

            }
            catch (Exception ex)
            {
                // Catch the exception message.
                String exceptionMessage = ex.Message;

            }

        } // End of the BeginSaveMapToFile method

        /// <summary>
        /// Save the map to a file.
        /// </summary>
        /// <param name="result"></param>
        private static void SaveMapToFile(IAsyncResult result)
        {
            // Create streams and a writer
            Stream fileStream = null;
            BinaryWriter binaryWriter = null;
            StorageContainer container = null;

            try
            {
                // Get the container
                container = staticStorageDevice.EndOpenContainer(result);

                // Check if the file exists and delete it if it does.
                if (container.FileExists(MapDataStorage.staticMapName))
                    container.DeleteFile(MapDataStorage.staticMapName);

                // Declare a variable for the file.
                fileStream = container.CreateFile(MapDataStorage.staticMapName);

                // Write to the file.
                binaryWriter = new BinaryWriter(fileStream);
                binaryWriter.Write(MapDataStorage.staticBlockMap3D.cubeSize);
                binaryWriter.Write(MapDataStorage.staticBlockMap3D.mapWidthX);
                binaryWriter.Write(MapDataStorage.staticBlockMap3D.mapDepthZ);
                binaryWriter.Write(MapDataStorage.staticBlockMap3D.mapHeightY);
                
                // Write each cube to the file
                for (int y = 0; y < MapDataStorage.staticBlockMap3D.mapHeightY; y++)
                {
                    for (int z = 0; z < MapDataStorage.staticBlockMap3D.mapDepthZ; z++)
                    {
                        for (int x = 0; x < MapDataStorage.staticBlockMap3D.mapWidthX; x++)
                        {
                            binaryWriter.Write(MapDataStorage.staticBlockMap3D.cubes[x, y, z]);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                // Catch the exception message.
                String exceptionMessage = ex.Message;
            }
            finally
            {
                // Close streams in the right order (Binary writer, file stream and container)
                if (binaryWriter != null)
                    binaryWriter.Close();
                if (fileStream != null)
                    fileStream.Close();
                // Dispose the container, to commit the data.
                if (container != null)
                    container.Dispose();

            }

        } // End of the SaveMapToFile method

        #endregion

        #region Open map methods

        /// <summary>
        /// This method loads a map from a file and returns a map object
        /// </summary>
        /// <param name="storageDevice"></param>
        /// <returns>A BlockMap3D object</returns>
        public static BlockMap3D LoadMapFromFile(StorageDevice storageDevice, string mapName)
        {
            // Create variables
            StorageContainer container = null;
            Stream fileStream = null;
            BinaryReader binaryReader = null;
            BlockMap3D tempBlockMap3D = null;
            string mapFileName = mapName + ".map";

            try
            {
                // Begin an asynchronous operation to open the container.
                IAsyncResult result = storageDevice.BeginOpenContainer(displayNameContainer, null, null);

                // Wait for the WaitHandle to become signaled.
                result.AsyncWaitHandle.WaitOne();

                // Get the container
                container = storageDevice.EndOpenContainer(result);

                // Close the wait handle.
                result.AsyncWaitHandle.Close();

                // Check to see whether the file exists.
                if (container.FileExists(mapFileName))
                {
                    // Open the file.
                    fileStream = container.OpenFile(mapFileName, FileMode.Open);

                    // Read from the file.
                    binaryReader = new BinaryReader(fileStream);
                    short cubeSize = binaryReader.ReadInt16();
                    short mapWidthX = binaryReader.ReadInt16();
                    short mapDepthZ = binaryReader.ReadInt16();
                    short mapHeightY = binaryReader.ReadInt16();

                    // Create a new block map 3D
                    tempBlockMap3D = new BlockMap3D(mapWidthX, mapDepthZ, mapHeightY, cubeSize);

                    // Read each cube from the file
                    for (int y = 0; y < mapHeightY; y++)
                    {
                        for (int z = 0; z < mapDepthZ; z++)
                        {
                            for (int x = 0; x < mapWidthX; x++)
                            {
                                tempBlockMap3D.cubes[x, y, z] = binaryReader.ReadInt16();
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                // Delete the file.
                String exceptionMessage = ex.Message;
            }
            finally
            {
                // Close streams in the right order (Buffered stream, Zip Stream and File Stream at last)
                if (binaryReader != null)
                    binaryReader.Close();
                if (fileStream != null)
                    fileStream.Close();
                
                // Dispose the container.
                if (container != null)
                    container.Dispose();
            }

            // Return the map
            return tempBlockMap3D;

        } // End of the LoadMapFromFile method

        /// <summary>
        /// This method gets all the map names from the map storage container.
        /// </summary>
        /// <param name="storageDevice"></param>
        /// <returns></returns>
        public static string[] GetMapNames(StorageDevice storageDevice)
        {
            // Create a string array to return and a storage container
            string[] mapNames = null;
            StorageContainer container = null;

            try
            {
                // Begin to open the container
                IAsyncResult result = storageDevice.BeginOpenContainer(displayNameContainer, null, null);

                // Wait for the WaitHandle to become signaled.
                result.AsyncWaitHandle.WaitOne();

                // Get the container
                container = storageDevice.EndOpenContainer(result);

                // Close the wait handle.
                result.AsyncWaitHandle.Close();

                // Get all the map names
                mapNames = container.GetFileNames();

            }
            catch (Exception ex)
            {
                // Catch the exception message.
                String exceptionMessage = ex.Message;
 
            }
            finally
            {
                // Dispose the container
                if (container != null)
                    container.Dispose();
            }

            // Return all the map names
            return mapNames;

        } // End of the GetMapNames method

        /// <summary>
        /// Check if a map name exist.
        /// </summary>
        /// <param name="storageDevice"></param>
        /// <param name="mapNameToCheck"></param>
        /// <returns></returns>
        public static bool CheckIfMapExist(StorageDevice storageDevice, string mapNameToCheck)
        {
            // A boolean to indicate if a map exist
            bool doMapExist = false;

            // Get all map names
            string[] mapNames = MapDataStorage.GetMapNames(storageDevice);

            if (mapNames == null)
                return false;

            // Loop through each mapName and indicate if the name exists
            foreach (string map in mapNames)
            {
                // Check if the name exists
                if (map.Split('.')[0] == mapNameToCheck)
                {
                    doMapExist = true;
                    break;
                }
            }

            // Return the boolean value
            return doMapExist;

        } // End of the CheckIfMapExist method.

        #endregion

        #region Delete map methods

        /// <summary>
        /// This method is used to delete a map
        /// </summary>
        /// <param name="storageDevice"></param>
        /// <param name="mapName"></param>
        public static void DeleteMap(StorageDevice storageDevice, string mapName)
        {
            // Create variables
            StorageContainer container = null;
            string mapFileName = mapName + ".map";

            try
            {
                // Begin an asynchronous operation to open the container.
                IAsyncResult result = storageDevice.BeginOpenContainer(displayNameContainer, null, null);

                // Wait for the WaitHandle to become signaled.
                result.AsyncWaitHandle.WaitOne();

                // Get the container
                container = storageDevice.EndOpenContainer(result);

                // Close the wait handle.
                result.AsyncWaitHandle.Close();

                // Check to see whether the file exists.
                if (container.FileExists(mapFileName))
                {
                    // Delete the file
                    container.DeleteFile(mapFileName);
                }
            }
            catch (Exception ex)
            {
                // Delete the file.
                String exceptionMessage = ex.Message;
            }
            finally
            {
                // Dispose the container.
                if (container != null)
                    container.Dispose();
            }

        } // End of DeleteMap method

        #endregion

        #region Storage device methods

        /// <summary>
        /// Set a a common storage device
        /// </summary>
        public static void SelectCommonStorageDevice()
        {
            // Ask the user to select a storage device.
            if (Program.commonStorageDevice == null || !Program.commonStorageDevice.IsConnected)
            {
                if (!Guide.IsVisible)
                {
                    try
                    {
                        // Start to show the selector for a storage device, files are saved to a common
                        // container for all player profiles.
                        AsyncCallback asyncCallback = new AsyncCallback(SetCommonStorageDevice);
                        StorageDevice.BeginShowSelector(asyncCallback, null);

                    }
                    catch (Exception exception)
                    {
                        String exMessage = exception.Message;
                    }
                }
            }

        } // End of the SetCommonStorageDevice method

        /// <summary>
        /// Set the common storage device
        /// </summary>
        /// <param name="result"></param>
        private static void SetCommonStorageDevice(IAsyncResult result)
        {

            try
            {
                // Get the storage device
                Program.commonStorageDevice = StorageDevice.EndShowSelector(result);
            }
            catch (Exception exception)
            {
                String exMessage = exception.Message;
            }

        } // End of the SetCommonStorageDevice method

        #endregion

    } // End of the class

} // End of the namespace
