﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.PlayerProfiles;
using AvatarBlockWar.Tools;
using AvatarBlockWar.DataStorage;
using MapObjects;
using ScreenManagement;
using CubeEngines;

namespace AvatarBlockWar.GameLogic
{
    /// <summary>
    /// This class represent game play logic for a build map game.
    /// </summary>
    public class BuildMapLogic : GamePlayLogic
    {
        #region Variables

        public SignedInGamer singleGamer;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new build map logic object
        /// </summary>
        /// <param name="content"></param>
        /// <param name="networkSession"></param>
        /// <param name="graphicsDevice"></param>
        /// <param name="controllingPlayer"></param>
        /// <param name="mapName"></param>
        public BuildMapLogic(ContentManager content, NetworkSession networkSession, GraphicsDevice graphicsDevice, PlayerIndex controllingPlayer, string mapName)
            : base(content, networkSession, graphicsDevice)
        {
            // Set the map name
            this.mapName = mapName;

            // Check if the map exists
            bool doMapExist = MapDataStorage.CheckIfMapExist(Program.commonStorageDevice, mapName);

            // Load the map if it exists
            if (doMapExist == true)
                this.currentBlockMap = MapDataStorage.LoadMapFromFile(Program.commonStorageDevice, mapName);
            else
                this.currentBlockMap = new BlockMap3D(500, 500, 200, 40); // Create a new map

            // Create a single gamer
            this.singleGamer = SignedInGamer.SignedInGamers[controllingPlayer];
            
            // Greate the player profile
            this.singleGamer.Tag = new HumanPlayerProfile(1, SignedInGamer.SignedInGamers[controllingPlayer].Gamertag, true, new Vector3(100, 200, 100), 
                new Vector3(0, 0, -1), this.standardViewPort, content, graphicsDevice, this.singleGamer);
            
            // Give the player a free flying camera.
            HumanPlayerProfile player = this.singleGamer.Tag as HumanPlayerProfile;
            player.freeFlyingCamera = true;

        } // End of the constructor

        #endregion

        #region Update

        /// <summary>
        /// Update the game logic
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Update(GameTime gameTime)
        {
            // Nothing to do here yet.

        } // End of the Update method

        #endregion

        #region Player input

        /// <summary>
        /// Handle player input
        /// </summary>
        /// <param name="gamer"></param>
        /// <param name="input"></param>
        /// <param name="playerIndex"></param>
        public override void HandlePlayerInput(LocalNetworkGamer gamer, InputState input, PlayerIndex playerIndex)
        {
            // Get the gamer
            HumanPlayerProfile singleGamer = SignedInGamer.SignedInGamers[playerIndex].Tag as HumanPlayerProfile;

            // Handle player input
            singleGamer.HandlePlayerInput(input, playerIndex, this.currentBlockMap, this.cubeEngine, this.audioManager);

        } // End of the HandlePlayerInput method

        #endregion

        #region Draw

        /// <summary>
        /// Draw in the build map logic
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="screenManager"></param>
        public override void Draw(GameTime gameTime, ScreenManager screenManager)
        {

            // Get the playerProfile
            HumanPlayerProfile player = this.singleGamer.Tag as HumanPlayerProfile;

            // Set the viewport to the players view port.
            screenManager.GraphicsDevice.Viewport = player.viewPort;

            // Draw solid cubes
            if (this.currentBlockMap != null)
                this.cubeEngine.DrawSolidCubes(screenManager.GraphicsDevice, player.camera.view, player.camera.projection);

            // Draw the sky last
            this.skyModel.Draw(player.camera.view, player.camera.projection);

            // Draw transparent cubes
            if (this.currentBlockMap != null)
                this.cubeEngine.DrawTransparentCubes(screenManager.GraphicsDevice, player.camera.view, player.camera.projection);

            // Draw the tool
            player.tools[player.currentTool].DrawTool(player.camera.view, player.camera.projection);

            // Create a SpriteBatch object
            SpriteBatch spriteBatch = screenManager.spriteBatch;

            // Start draw in 2D
            spriteBatch.Begin();

            // Draw the crosshair
            spriteBatch.Draw(crosshairTexture, new Vector2((player.viewPort.Width / 2) - (crosshairTexture.Width / 2),
                (player.viewPort.Height / 2) - (crosshairTexture.Height / 2)), Color.White);

            // Draw cube menus
            player.cubeSelectionMenu.DrawMenus(spriteBatch, this.gameFont, this.cubeEngine);

            // Draw the framerate
            spriteBatch.DrawString(screenManager.font, "Frames/second: " + GameEngine.frameRate, new Vector2(20, 20), Color.Black);

            spriteBatch.End();

            // Reset the viewport
            screenManager.GraphicsDevice.Viewport = this.standardViewPort;
      
        } // End of the draw method.

        #endregion

    } // End of the class

} // End of the namespace
