﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Net;
using AvatarBlockWar.DataStorage;
using AvatarBlockWar.PlayerProfiles;
using CubeEngines;
using MapObjects;
using ScreenManagement;
using AudioEngine;

namespace AvatarBlockWar.GameLogic
{
    /// <summary>
    /// This class represent gameplay logic.
    /// </summary>
    public abstract class GamePlayLogic : IAudioEmitter
    {

        #region Instance variables

        // Network session and standard viewport
        public NetworkSession networkSession;
        public Viewport standardViewPort;

        // A block map and a cube engine
        public BlockMap3D currentBlockMap = null;
        public CubeEngine cubeEngine;

        // A audio manager
        public AudioManager audioManager;

        // Models
        public Sky skyModel;

        // Crosshair texture
        public Texture2D crosshairTexture;

        // Game font
        public SpriteFont gameFont;

        // Map name
        public string mapName;

        // A list of player profiles
        public List<PlayerProfile> playerProfiles;

        // End game logic
        public int numberOfLives;
        public int gameTimeLeft;
        public bool gameHasEnded = false;
        public string winningTeam = "";

        #endregion

        #region Audio properties

        /// <summary>
        /// Gets or sets the 3D position of the entity.
        /// </summary>
        private Vector3 position;
        public Vector3 Position
        {
            get { return position; }
            set { position = value; }
        }

        /// <summary>
        /// Gets or sets which way the entity is facing.
        /// </summary>
        private Vector3 forward;
        public Vector3 Forward
        {
            get { return forward; }
            set { forward = value; }
        }

        /// <summary>
        /// Gets or sets the orientation of this entity.
        /// </summary>
        private Vector3 up;
        public Vector3 Up
        {
            get { return up; }
            set { up = value; }
        }

        /// <summary>
        /// Gets or sets how fast this entity is moving.
        /// </summary>
        private Vector3 velocity;
        public Vector3 Velocity
        {
            get { return velocity; }
            protected set { velocity = value; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Create new game play logic
        /// </summary>
        /// <param name="networkSession"></param>
        public GamePlayLogic(ContentManager content, NetworkSession networkSession, GraphicsDevice graphicsDevice)
        {
            // Set values for instance variables
            this.networkSession = networkSession;
            this.standardViewPort = new Viewport(0, 0, 1280, 720);
            this.numberOfLives = Program.numberOfLives;
            this.gameTimeLeft = Program.totalGameTimeMilliseconds;
  
            // Load the font
            this.gameFont = content.Load<SpriteFont>("Fonts/gamefont");

            // Load models
            this.skyModel = content.Load<Sky>("GameTextures/sky");

            // Load textures
            this.crosshairTexture = content.Load<Texture2D>("GameTextures/crosshair");

            // Create the cube engine
            this.cubeEngine = new InstancedCubeEngine(graphicsDevice, content);

        } // End of the constructor

        #endregion

        #region Dispose

        /// <summary>
        /// Dispose the audio manager
        /// </summary>
        public void DisposeAudio()
        {
            // Dispose the audio manager to stop all sounds.
            this.audioManager.Dispose(true);

        } // End of the DisposeAudio method

        #endregion

        #region Update

        public abstract void Update(GameTime gameTime);

        /// <summary>
        /// Check if a game has ended
        /// </summary>
        /// <param name="gameTime"></param>
        public virtual void CheckIfGameHasEnded(GameTime gameTime)
        {
            // Just return if the game already has ended
            if (this.gameHasEnded == true)
                return;

            // Check if the game time has run out
            if (this.gameTimeLeft <= 0)
            {
                this.gameHasEnded = true;
                this.winningTeam = "Survivors";
            }

            // Check if the survivors have lost all of there lives.
            if (this.numberOfLives <= 0)
            {
                this.gameHasEnded = true;
                this.winningTeam = "Attackers";
            }

        } // End of the CheckIfGameHasEnded method

        #endregion

        #region Player input

        public abstract void HandlePlayerInput(LocalNetworkGamer gamer, InputState input, PlayerIndex playerIndex);

        #endregion

        #region Draw

        public abstract void Draw(GameTime gameTime, ScreenManager screenManager);

        #endregion

    } // End of the class

} // End of the namespace
