﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.PlayerProfiles;
using AvatarBlockWar.Tools;
using AvatarBlockWar.DataStorage;
using MapObjects;
using ScreenManagement;
using CubeEngines;
using AudioEngine;

namespace AvatarBlockWar.GameLogic
{
    /// <summary>
    /// This class represent game play logic for a local network game.
    /// </summary>
    public class LocalGameLogic : GamePlayLogic
    {
        #region Variables

        // Computer player profiles
        public List<ComputerPlayerProfile> computerPlayerProfiles;

        // Four viewports
        List<Viewport> viewPorts = new List<Viewport>(4);

        // Constant x and y offsets
        const int X_OFFSET = 128;
        const int Y_OFFSET = 72;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new local game logic
        /// </summary>
        /// <param name="content"></param>
        /// <param name="networkSession"></param>
        /// <param name="graphicsDevice"></param>
        public LocalGameLogic(ContentManager content, NetworkSession networkSession, GraphicsDevice graphicsDevice, 
            string mapName)
            : base(content, networkSession, graphicsDevice)
        {

            // Create a new audio manager
            this.audioManager = new AudioManager(content, 1500f, 0.5f);

            // Set the map name
            this.mapName = mapName;

            // Check if the map exists
            bool doMapExist = MapDataStorage.CheckIfMapExist(Program.commonStorageDevice, mapName);

            // Load the map if it exists
            if (doMapExist == true)
                this.currentBlockMap = MapDataStorage.LoadMapFromFile(Program.commonStorageDevice, mapName);
            else
                this.currentBlockMap = new BlockMap3D(500, 500, 200, 40); // Create a new map

            // Calculate the number of local gamers in the session
            int numberOfHumanGamers = networkSession.LocalGamers.Count;
            
            // Create viewports
            switch (numberOfHumanGamers)
            {
                case 1:
                    viewPorts.Add(this.standardViewPort);
                    break;
                case 2:
                    viewPorts.Add(new Viewport(0, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height));
                    viewPorts.Add(new Viewport(this.standardViewPort.Width / 2, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height));
                    break;
                case 3:
                    viewPorts.Add(new Viewport(0, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    viewPorts.Add(new Viewport(this.standardViewPort.Width / 2, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    viewPorts.Add(new Viewport(0, this.standardViewPort.Height / 2, this.standardViewPort.Width, this.standardViewPort.Height / 2));
                    break;
                case 4:
                    viewPorts.Add(new Viewport(0, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    viewPorts.Add(new Viewport(this.standardViewPort.Width / 2, 0, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    viewPorts.Add(new Viewport(0, this.standardViewPort.Height / 2, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    viewPorts.Add(new Viewport(this.standardViewPort.Width / 2, this.standardViewPort.Height / 2, this.standardViewPort.Width / 2, this.standardViewPort.Height / 2));
                    break;
            }

            // Create an array with player profiles
            this.playerProfiles = new List<PlayerProfile>(16);

            // Create human player profiles (Max 4)
            for (int i = 0; i < this.networkSession.LocalGamers.Count; i++)
            {
                // Get the local network gamer
                LocalNetworkGamer gamer = this.networkSession.LocalGamers[i];

                // Get a start position
                Vector3 startPosition = StaticPlayerProfileMethods.getRandomStartPosition();

                // Create a new player profile for the gamer
                gamer.Tag = new HumanPlayerProfile(gamer.Id, gamer.Gamertag, true, startPosition, new Vector3(0, 0, -1), this.viewPorts[i], content, graphicsDevice, gamer);
                this.playerProfiles.Add(gamer.Tag as HumanPlayerProfile);
            }

            // Create the list of computer profiles
            this.computerPlayerProfiles = new List<ComputerPlayerProfile>(12);

            // Create computer player profiles
            for (int c = 0; c < 12; c++)
            {
                // Get a start position
                Vector3 startPosition = StaticPlayerProfileMethods.getRandomStartPosition();

                // Add a computer player
                computerPlayerProfiles.Add(new ComputerPlayerProfile(122, "Computer AI" + c, false, startPosition, new Vector3(0, 0, -1), content, graphicsDevice));
                this.playerProfiles.Add(computerPlayerProfiles[c]);
            }

            // Start the background sound
            this.audioManager.Play3DSound("summer", true, this);

        } // End of the constructor

        #endregion

        #region Update

        /// <summary>
        /// Update the game logic
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Update(GameTime gameTime)
        {

            // Update the audio manager
            this.audioManager.Update(gameTime);

            // Update each human player profile
            foreach (LocalNetworkGamer gamer in this.networkSession.LocalGamers)
            {
                // Get the player profile for the local gamer
                HumanPlayerProfile player = gamer.Tag as HumanPlayerProfile;

                // Update the avatar model
                player.playerModel.UpdateAvatar(gameTime, player.playerPosition, player.AvatarRotation);

                // Check if we should add a bullet
                if (player.isDead == false)
                    StaticPlayerProfileMethods.AddBullet(gameTime, player, player.lookUpDown, new Vector3(0, player.camera.cameraHeightOverPlayerFeets, 0), this.audioManager);

                // Update the audio
                player.UpdateAudio(gameTime, this.audioManager);

                // Check if the player has been hit by a bullet
                if(player.isDead == false)
                    player.CheckIfPlayersIsHitByBullet(this.playerProfiles, this);

                // Update the spawn time for a dead soldier
                player.UpdateSpawnTime(gameTime);
            
            }

            // Update each computer player profile
            foreach (ComputerPlayerProfile computerPlayer in this.computerPlayerProfiles)
            {
                // Handle computer player movement
                if(computerPlayer.isDead == false)
                    computerPlayer.HandlePlayerInput(this.currentBlockMap, this.cubeEngine, this.playerProfiles);

                // Update the avatar model
                computerPlayer.playerModel.UpdateAvatar(gameTime, computerPlayer.playerPosition, computerPlayer.AvatarRotation);

                // Check if we should add a bullet
                if(computerPlayer.isDead == false)
                    StaticPlayerProfileMethods.AddBullet(gameTime, computerPlayer, Vector3.Zero, new Vector3(0, 40, 0), this.audioManager);

                // Update the audio
                computerPlayer.UpdateAudio(gameTime, this.audioManager);

                // Check if the player has been hit by a bullet
                if(computerPlayer.isDead == false)
                    computerPlayer.CheckIfPlayersIsHitByBullet(this.playerProfiles, this);

                // Update the spawn time for a dead soldier
                computerPlayer.UpdateSpawnTime(gameTime);
            }

            // Decrease the game time
            if (this.gameHasEnded == false)
            {
                this.gameTimeLeft -= gameTime.ElapsedGameTime.Milliseconds;
            }

            // Check if the game has ended
            CheckIfGameHasEnded(gameTime);

        } // End of the Update method

        #endregion

        #region Player input

        /// <summary>
        /// Handle player input
        /// </summary>
        /// <param name="gamer"></param>
        /// <param name="input"></param>
        /// <param name="playerIndex"></param>
        public override void HandlePlayerInput(LocalNetworkGamer gamer, InputState input, PlayerIndex playerIndex)
        {
            // Get the gamer
            HumanPlayerProfile player = gamer.Tag as HumanPlayerProfile;

            // Handle player input
            if(player.isDead == false)
                player.HandlePlayerInput(input, playerIndex, this.currentBlockMap, this.cubeEngine, this.audioManager);

        } // End of the HandlePlayerInput method

        #endregion

        #region Draw

        /// <summary>
        /// Draw the local game logic
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="screenManager"></param>
        public override void Draw(GameTime gameTime, ScreenManager screenManager)
        {

            // Draw to the viewport for each local gamer
            foreach(LocalNetworkGamer gamer in this.networkSession.LocalGamers)
            {

                // Get the player profile for the local gamer
                HumanPlayerProfile player = gamer.Tag as HumanPlayerProfile;

                // Set the viewport to the players view port.
                screenManager.GraphicsDevice.Viewport = player.viewPort;

                // Create a bounding frustum
                BoundingFrustum boundingFrustum = new BoundingFrustum(player.camera.view * player.camera.projection);

                // Draw solid cubes
                if (this.currentBlockMap != null)
                    this.cubeEngine.DrawSolidCubes(screenManager.GraphicsDevice, player.camera.view, player.camera.projection);

                // Draw avatar models for all gamers except the current player
                foreach (LocalNetworkGamer otherGamer in this.networkSession.LocalGamers)
                {
                    if (otherGamer.Id != player.playerId)
                    {
                        // Get the player profile for the gamer
                        HumanPlayerProfile otherPlayer = otherGamer.Tag as HumanPlayerProfile;
                        otherPlayer.playerModel.DrawAvatar(player.camera.view, player.camera.projection);
                    }
                }

                // Draw computer player models
                foreach (ComputerPlayerProfile computerPlayer in this.computerPlayerProfiles)
                {
                    // Only draw avatars that are included in the bounding frustum.
                    Vector3 pos = computerPlayer.playerPosition;
                    BoundingBox ComputerPlayerBox = new BoundingBox(new Vector3(pos.X - 10, pos.Y, pos.Z - 10), new Vector3(pos.X + 10, pos.Y + 50, pos.Z + 10));
                    if(boundingFrustum.Intersects(ComputerPlayerBox))
                        computerPlayer.playerModel.DrawAvatar(player.camera.view, player.camera.projection);
                }

                // Draw the sky before transparent cubes but after all other things
                this.skyModel.Draw(player.camera.view, player.camera.projection);

                // Draw transparent cubes
                if (this.currentBlockMap != null)
                    this.cubeEngine.DrawTransparentCubes(screenManager.GraphicsDevice, player.camera.view, player.camera.projection);

                // Draw the tool
                player.tools[player.currentTool].DrawTool(player.camera.view, player.camera.projection);

                // Draw the shot effect
                player.tools[player.currentTool].playerShotEffect.DrawEffect(gameTime, player.camera.view, player.camera.projection);

                // Draw enemy shot effect and blood effect for computer players
                foreach (ComputerPlayerProfile computerPlayer in this.computerPlayerProfiles)
                {
                    computerPlayer.playerModel.shotEffect.DrawEffect(gameTime, player.camera.view, player.camera.projection);
                    computerPlayer.playerModel.bloodEffect.DrawEffect(gameTime, player.camera.view, player.camera.projection);
                }

                // Create a SpriteBatch object
                SpriteBatch spriteBatch = screenManager.spriteBatch;

                if (player.isDead == false)
                {
                    // Start draw in 2D
                    spriteBatch.Begin();

                    // Draw the crosshair
                    spriteBatch.Draw(crosshairTexture, new Vector2((player.viewPort.Width / 2) - (crosshairTexture.Width / 2),
                        (player.viewPort.Height / 2) - (crosshairTexture.Height / 2)), Color.White);

                    // Draw cube menus
                    player.cubeSelectionMenu.DrawMenus(spriteBatch, this.gameFont, this.cubeEngine);

                    // Draw the framerate
                    spriteBatch.DrawString(this.gameFont, "Frames/second: " + GameEngine.frameRate, new Vector2(X_OFFSET, Y_OFFSET), Color.White);

                    // Draw the health
                    spriteBatch.DrawString(this.gameFont, "Health: " + player.health + " %", new Vector2(X_OFFSET, Y_OFFSET + 20), Color.White);

                    // Draw kills and deaths
                    spriteBatch.DrawString(this.gameFont, "K: " + player.kills, new Vector2(X_OFFSET, Y_OFFSET + 40), Color.White);
                    spriteBatch.DrawString(this.gameFont, "D: " + player.deaths, new Vector2(X_OFFSET, Y_OFFSET + 60), Color.White);

                    // Draw the game time left
                    int minutes = this.gameTimeLeft / 60000;
                    string gameTimeLeft = minutes + ":" + String.Format("{0:00}", (this.gameTimeLeft - (minutes * 60000)) / 1000);
                    spriteBatch.DrawString(this.gameFont, gameTimeLeft, new Vector2(player.viewPort.Width - (this.gameFont.MeasureString(gameTimeLeft).X * 1f) - X_OFFSET, Y_OFFSET), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0);

                    // Draw the number of lives left
                    string lifesLeftString = "Lives left: " + this.numberOfLives.ToString();
                    spriteBatch.DrawString(this.gameFont, lifesLeftString, new Vector2(player.viewPort.Width - (this.gameFont.MeasureString(lifesLeftString).X * 1f) - X_OFFSET, Y_OFFSET + 20), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0);

                    spriteBatch.End();
                }
                else if (player.isDead == true)
                {
                    // Darken down the viewport under the text.
                    screenManager.FadeBackBufferToBlack(0.5f);

                    // Draw a spawn screen if the player is dead.
                    spriteBatch.Begin();

                    // Killed by string
                    String killedByString = "You were killed by " + player.killedByGamerTag;

                    // Time string
                    String timeString = "Respawn in " + (player.spawnCountDown / 1000) + " seconds";

                    // Draw the killed by name centered on the screen
                    SpriteFont killedByFont = this.gameFont;
                    Vector2 textPosition = new Vector2(player.viewPort.Width / 2, player.viewPort.Height / 2);
                    Vector2 textOrigin = killedByFont.MeasureString(killedByString) / 2;
                    Color killedStringColor = new Color(200, 0, 0);
                    float killedByTextScale = 1.00f;

                    // Draw the killed by name
                    spriteBatch.DrawString(killedByFont, killedByString, textPosition, killedStringColor, 0,
                                           textOrigin, killedByTextScale, SpriteEffects.None, 0);

                    // Draw the respawn time
                    spriteBatch.DrawString(killedByFont, timeString, textPosition + new Vector2(0, 40), killedStringColor, 0,
                                            killedByFont.MeasureString(timeString) / 2, killedByTextScale, SpriteEffects.None, 0);

                    spriteBatch.End();
                }

            } // End of the foreach(LocalNetworkGamer gamer in this.networkSession.LocalGamers) loop

            // Reset the viewport
            screenManager.GraphicsDevice.Viewport = this.standardViewPort;

        } // End of the draw method.

        #endregion

    } // End of the class

} // End of the namespace
