﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using CubeEngines;

namespace MapObjects
{
    /// <summary>
    /// This class represent menus to handle cubeType selection
    /// </summary>
    public class CubeSelectionMenu
    {
        #region Variables

        // Block selection menu
        public Int16[] blockSelectionMenu = new Int16[] { 1, 2, 3, 4 };
        public Int32[] selectedBlock = new Int32[3] { 0, 0, 0 };
        public Int16 selectionPosition = 0;
        public Int16 selectedBlockType = 1;

        // Big cube menu
        public bool isBigCubeMenuVisible = false;
        private byte[] selectionBigCubeMenu = new byte[] { 0, 0 };
        private const byte NUMBER_OF_BLOCKS_PER_ROW = 5;
        private const byte NUMBER_OF_BLOCK_ROWS = 3;

        // Textures
        private Texture2D backgroundTexture;
        private Texture2D selectionTexture;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new CubeSelection menu
        /// </summary>
        /// <param name="content"></param>
        public CubeSelectionMenu(ContentManager content)
        {
            // Load textures
            this.backgroundTexture = content.Load<Texture2D>("MenuTextures/darkConcrete");
            this.selectionTexture = content.Load<Texture2D>("MenuTextures/selectionRectangle");

        } // End of the constructor

        #endregion

        #region Methods

        public void HandleMenuInput(KeyboardState currentKeyState, KeyboardState previousKeyState,
            GamePadState currentPadState, GamePadState previousPadState, CubeEngine cubeEngine)
        {

            // Change the selected block
            if (currentPadState.IsButtonDown(Buttons.DPadUp) && previousPadState.IsButtonUp(Buttons.DPadUp) ||
                currentKeyState.IsKeyDown(Keys.Up) && previousKeyState.IsKeyUp(Keys.Up))
            {
                if (this.selectionPosition > 0)
                {
                    this.selectionPosition--;
                    this.selectedBlockType = this.blockSelectionMenu[this.selectionPosition];
                }
            }
            if (currentPadState.IsButtonDown(Buttons.DPadDown) && previousPadState.IsButtonUp(Buttons.DPadDown) || 
                currentKeyState.IsKeyDown(Keys.Down) && previousKeyState.IsKeyUp(Keys.Down))
            {
                if (this.selectionPosition < 3)
                {
                    this.selectionPosition++;
                    this.selectedBlockType = this.blockSelectionMenu[this.selectionPosition];
                }
            }

            // Should we show the big cube menu
            if (currentPadState.IsButtonDown(Buttons.B) && previousPadState.IsButtonUp(Buttons.B) || 
                currentKeyState.IsKeyDown(Keys.B) && previousKeyState.IsKeyUp(Keys.B))
            {
                if (this.isBigCubeMenuVisible == false)
                    this.isBigCubeMenuVisible = true;
                else
                    this.isBigCubeMenuVisible = false;
            }

            // Respond to input on the big cube menu.
            if (this.isBigCubeMenuVisible == true)
            {
                // Change the selection rectangle up, down, right or left.
                if (currentPadState.ThumbSticks.Left.Y > 0.25f && previousPadState.ThumbSticks.Left.Y < 0.25f ||
                    currentKeyState.IsKeyDown(Keys.W) && previousKeyState.IsKeyUp(Keys.W))
                {
                    // Y-position--
                    if (this.selectionBigCubeMenu[1] > 0)
                        this.selectionBigCubeMenu[1]--;
                }
                if (currentPadState.ThumbSticks.Left.X < -0.25f && previousPadState.ThumbSticks.Left.X > -0.25f ||
                    currentKeyState.IsKeyDown(Keys.A) && previousKeyState.IsKeyUp(Keys.A))
                {
                    // X-position--
                    if (this.selectionBigCubeMenu[0] > 0)
                        this.selectionBigCubeMenu[0]--;
                }
                if (currentPadState.ThumbSticks.Left.X > 0.25f && previousPadState.ThumbSticks.Left.X < 0.25f || 
                    currentKeyState.IsKeyDown(Keys.D) && previousKeyState.IsKeyUp(Keys.D))
                {
                    // X-position++
                    if (this.selectionBigCubeMenu[0] < (NUMBER_OF_BLOCKS_PER_ROW - 1))
                        this.selectionBigCubeMenu[0]++;
                }
                if (currentPadState.ThumbSticks.Left.Y < -0.25f && previousPadState.ThumbSticks.Left.Y > -0.25f || 
                    currentKeyState.IsKeyDown(Keys.S) && previousKeyState.IsKeyUp(Keys.S))
                {
                    // Y-position++
                    int maxColumn = (cubeEngine.cubeTypes.Length / NUMBER_OF_BLOCKS_PER_ROW) > 0 ? (cubeEngine.cubeTypes.Length / NUMBER_OF_BLOCKS_PER_ROW) - 1 : 0;
                    if (this.selectionBigCubeMenu[1] < maxColumn)
                        this.selectionBigCubeMenu[1]++;
                }

                // Change the selected block in the selection menu to the selected block in the big menu.
                if (currentPadState.IsButtonDown(Buttons.A) && previousPadState.IsButtonUp(Buttons.A) || 
                    currentKeyState.IsKeyDown(Keys.M) && previousKeyState.IsKeyUp(Keys.M))
                {
                    Int16 choosenBlock = (Int16)((this.selectionBigCubeMenu[1] * NUMBER_OF_BLOCKS_PER_ROW) + this.selectionBigCubeMenu[0] + 1);

                    blockSelectionMenu[this.selectionPosition] = choosenBlock;
                    this.selectedBlockType = choosenBlock;
                }
            }

        } // End of the HandleMenuInput method

        #endregion

        #region Draw

        /// <summary>
        /// Draw menus on the screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        /// <param name="cubeTypes"></param>
        /// <param name="selectionPosition"></param>
        public void DrawMenus(SpriteBatch spriteBatch, SpriteFont font, CubeEngine cubeEngine)
        {
            // Draw the block selection menu
            int yPosition = 251;
            for (int i = 0; i < blockSelectionMenu.Length; i++)
            {
                spriteBatch.Draw(cubeEngine.cubeTextureMap, new Rectangle(72, yPosition, 64, 64),
                    new Rectangle((int)(cubeEngine.cubeTypes[blockSelectionMenu[i]].cubeTextureOffset.X * 4096), 
                        (int)(cubeEngine.cubeTypes[blockSelectionMenu[i]].cubeTextureOffset.Y * 4096), 128, 128), Color.White);

                yPosition += 65;
            }

            // Draw the selection rectangle
            yPosition = 249 + (65 * this.selectionPosition);
            spriteBatch.Draw(this.selectionTexture, new Rectangle(71, yPosition, 66, 66), Color.White);

            // Draw the big cube menu
            if (this.isBigCubeMenuVisible == true)
            {
                // Draw the background texture
                spriteBatch.Draw(this.backgroundTexture, new Rectangle(256, 72, 800, 550), Color.White);

                // Draw the cubes
                int xPos = 276;
                int yPos = 92;
                int xCounter = 0;

                // Calculate the start number
                int startColumn = this.selectionBigCubeMenu[1] - (NUMBER_OF_BLOCK_ROWS - 1);
                if (startColumn < 0)
                    startColumn = 0;
                int startNumber = startColumn * NUMBER_OF_BLOCKS_PER_ROW + 1;
                int endNumber = startNumber + (NUMBER_OF_BLOCKS_PER_ROW * NUMBER_OF_BLOCK_ROWS);
                if (endNumber > cubeEngine.cubeTypes.Length)
                    endNumber = cubeEngine.cubeTypes.Length;

                // Draw the cubes.
                for (int t = startNumber; t < endNumber; t++)
                {
                    spriteBatch.DrawString(font, cubeEngine.cubeTypes[t].cubeName, new Vector2(xPos, yPos), Color.White, 0, Vector2.Zero, 0.8f, SpriteEffects.None, 0);
                    spriteBatch.Draw(cubeEngine.cubeTextureMap, new Rectangle(xPos, yPos + 25, 128, 128), 
                        new Rectangle((int)(cubeEngine.cubeTypes[t].cubeTextureOffset.X * 4096),
                        (int)(cubeEngine.cubeTypes[t].cubeTextureOffset.Y * 4096), 128, 128), Color.White);

                    xCounter++;
                    xPos += 156;

                    if (xCounter == NUMBER_OF_BLOCKS_PER_ROW)
                    {
                        xCounter = 0;
                        xPos = 276;
                        yPos += 160;
                    }
                }

                // Draw the selection rectangle
                int ColumnNumber = this.selectionBigCubeMenu[1] > (NUMBER_OF_BLOCK_ROWS - 1) ? (NUMBER_OF_BLOCK_ROWS - 1) : this.selectionBigCubeMenu[1];
                xPos = 274 + (156 * this.selectionBigCubeMenu[0]);
                yPos = 115 + (160 * ColumnNumber);
                spriteBatch.Draw(this.selectionTexture, new Rectangle(xPos, yPos, 130, 130), Color.White);
            }

        } // End of the DrawMenus method

        #endregion

    } // End of the class

} // End of the namespace
