﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using AvatarBlockWar.AvatarModels;
using AvatarBlockWar.Tools;
using AvatarBlockWar.ParticleEffects;
using MapObjects;
using CubeEngines;

namespace AvatarBlockWar.PlayerProfiles
{
    /// <summary>
    /// This class represent a computer player profile
    /// </summary>
    public class ComputerPlayerProfile : PlayerProfile
    {
        #region Variables

        // Turn around counters (number of frames)
        public int turnAroundCounter = 0;
        public int turnAroundDelay = 600; // 10 seconds (60 frames/sec * 10 seconds)

        // Walk and stand counters (number of frames)
        public int walkStandCounter = 0;
        public int walkStandDuration = 600;
        public bool isWalking = false;

        // A random generator
        Random rndGenerator = new Random();

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new computer player profile
        /// </summary>
        /// <param name="playerId"></param>
        /// <param name="playerName"></param>
        /// <param name="isSurvivor"></param>
        /// <param name="playerPosition"></param>
        /// <param name="playerDirection"></param>
        /// <param name="content"></param>
        public ComputerPlayerProfile(byte playerId, string playerName, bool isSurvivor, Vector3 playerPosition, Vector3 playerDirection, 
            ContentManager content, GraphicsDevice graphicsDevice)
            : base(playerId, playerName, isSurvivor, playerPosition, playerDirection)
        {
            // Create the avatar model
            this.playerModel = new AvatarModel(content, graphicsDevice);
            this.playerModel.LoadRandomAvatar();

            // Set the player speed
            this.playerSpeed = 0.5f;

            this.tools = new Tool[1];
            this.tools[0] = new AssaultRifle(content.Load<Model>("Tools/AssaultRifle/weapon"), new AssaultRifleShotEffect(content, graphicsDevice));

            // Set the initial value for the turn around counter.
            this.turnAroundCounter = this.turnAroundDelay;

        } // End of the constructor

        #endregion

        #region Methods

        /// <summary>
        /// Handle computer player movement
        /// </summary>
        /// <param name="blockMap3D"></param>
        /// <param name="cubeTypes"></param>
        public void HandlePlayerInput(BlockMap3D blockMap3D, CubeEngine cubeEngine, List<PlayerProfile> playerProfiles)
        {
            // Set the previous player position.
            this.previousPlayerPosition = this.playerPosition;

            // ***********************************************************************************************
            // Check if the computer player should shot, if he is shooting then he will stand still and shot,
            // he will not walk, not turn around and so forth.
            // ***********************************************************************************************
            
            // Create a ray for the computer player
            Ray sightRay = new Ray(new Vector3(this.playerPosition.X, this.playerPosition.Y + 42, this.playerPosition.Z), this.playerDirection);
            PlayerProfile enemyInSight = null;

            // Check if the player has another player in sight.
            foreach (PlayerProfile enemy in playerProfiles)
            {
                // Just continue if the enemy player profile is the same as the player profile, 
                // do not shoot players on the same team and do not shot dead players.
                if (this.playerName == enemy.playerName || enemy.isDead == true || enemy.isSurvivor == false)
                    continue;

                // Create a bounding box for the enemy
                Vector3 enemyMinVector = new Vector3(enemy.playerPosition.X - 10, enemy.playerPosition.Y, enemy.playerPosition.Z - 10);
                Vector3 enemyMaxVector = new Vector3(enemy.playerPosition.X + 10, enemy.playerPosition.Y + 50, enemy.playerPosition.Z + 10);
                BoundingBox enemyBox = new BoundingBox(enemyMinVector, enemyMaxVector);

                if (enemyBox.Intersects(sightRay) != null && enemyBox.Intersects(sightRay) < 200f)
                {
                    // Check if there is a block in the way
                    int numberOfBlocks = 200 / blockMap3D.cubeSize;
                    Vector3 startPosition = this.playerPosition;
                    bool blockCollision = false;

                    for (int i = 0; i < numberOfBlocks; i++)
                    {
                        // Increase the startposition by 40
                        startPosition = startPosition + (this.playerDirection * 40);

                        // Get the block for the current position
                        int xTile = (int)startPosition.X / blockMap3D.cubeSize;
                        int zTile = (int)startPosition.Z / blockMap3D.cubeSize;
                        int yTile = (int)startPosition.Y / blockMap3D.cubeSize;

                        if (xTile >= blockMap3D.mapWidthX)
                            xTile = blockMap3D.mapWidthX - 1;
                        if (zTile >= blockMap3D.mapDepthZ)
                            zTile = blockMap3D.mapDepthZ - 1;
                        if (yTile >= blockMap3D.mapHeightY)
                            yTile = blockMap3D.mapHeightY - 1;
                        if (xTile <= 0)
                            xTile = 0;
                        if (zTile <= 0)
                            zTile = 0;
                        if (yTile <= 0)
                            yTile = 0;

                        if (blockMap3D.cubes[xTile, yTile, zTile] != -1)
                        {
                            blockCollision = true;
                        }

                    }

                    // Get the player to shot at if there isn´t a block in the way
                    if (blockCollision == false)
                    {
                        enemyInSight = enemy;
                        break;
                    }
                }
            }

            // Set a turn amount
            float turnAmount = 0;

            // Check if we should add a shot
            if (enemyInSight != null)
            {
                //// Turn to towards the enemy
                //Vector3 shootingDirection = enemyInSight.playerPosition - this.playerPosition;
                //shootingDirection.Normalize();

                //Vector3 destinationRightVector = Vector3.Cross(shootingDirection, Vector3.Up);
                //float forwardDot = Vector3.Dot(this.playerDirection, shootingDirection);
                //float rightDot = Vector3.Dot(this.playerDirection, destinationRightVector);

                //// Make sure we stay in range no matter what, so Acos doesn't fail later 
                //forwardDot = MathHelper.Clamp(forwardDot, -1.0f, 1.0f);

                //float angleBetween = (float)Math.Acos((float)forwardDot);

                //if (rightDot < 0.0f)
                //    angleBetween *= -1.0f;

                //// Set the turnAmount and the player direction
                //turnAmount = angleBetween;
                //this.playerDirection = shootingDirection;

                // Add a shot
                this.isShooting = true;

                // Play the shooting animation
                this.playerModel.SetAnimation(9);

            }
            else
            {
                this.isShooting = false;
            }

            // Check if the player should walk or stand still
            if (this.isShooting == false)
            {
                if (this.walkStandCounter < 0)
                {
                    // Calculate a random number
                    double rndValue = this.rndGenerator.NextDouble();

                    // Make the player stand still in 20 % of all cases
                    if (rndValue < 0.2f)
                    {
                        this.isWalking = false;
                        this.playerModel.SetRandomIdleAnimation();
                    }
                    else
                    {
                        // Set the boolean to indicate the player is shooting
                        this.isWalking = true;
                        
                    }

                    // Reset the walkStandCounter
                    this.walkStandCounter = this.walkStandDuration;
                }
                else
                {
                    if (this.isWalking == true)
                    {
                        // Move the computer player in the current direction
                        this.playerPosition += this.playerDirection * this.playerSpeed;

                        // Set the walking animation
                        this.playerModel.SetAnimation(4);
                    }

                    // Decrease the counter
                    this.walkStandCounter--;
                }
            }

            // Make a turn if the turnAroundCounter has reached zero and if the player not is shooting
            if (this.turnAroundCounter < 0 && this.isShooting == false)
            {
                // Calculate a random value
                float rndValue = (float)(this.rndGenerator.NextDouble() * 180) - 90;

                // Reset the turnAroundCounter
                this.turnAroundCounter = this.turnAroundDelay;

                turnAmount = MathHelper.ToRadians(rndValue);
                this.playerDirection = Vector3.Transform(this.playerDirection, Matrix.CreateFromAxisAngle(Vector3.Up, turnAmount));

            }
            else
            {
                this.turnAroundCounter--;
            }

            // Check for collisions
            this.CheckIfPlayerIsOutsideTheMap(blockMap3D);
            this.PlayerCollisionWithCube(blockMap3D, 50);

            // Update player model rotation
            UpdatePlayerRotation(turnAmount);

        } // End of the HandlePlayerInput method.

        #endregion

    } // End of the class

} // End of the namespace
