﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.AvatarModels;
using AvatarBlockWar.Tools;
using AvatarBlockWar.ParticleEffects;
using Cameras;
using ScreenManagement;
using CubeEngines;
using MapObjects;
using AudioEngine;

namespace AvatarBlockWar.PlayerProfiles
{
    /// <summary>
    /// This class represent a human player profile
    /// </summary>
    public class HumanPlayerProfile : PlayerProfile
    {
        #region Variables

        // Camera and viewport
        public Camera camera;
        public Viewport viewPort;

        // Variables to handle input
        public Vector3 lookUpDown;
        public bool freeFlyingCamera = false;

        // Add and remove blocks (Cube selection menus)
        public CubeSelectionMenu cubeSelectionMenu;
        public Vector3 addBlockVector = Vector3.Zero;

        // Input states
        public MouseState previousMouseState;

        // Turn amounts
        float turnLeftRightPC = MathHelper.ToRadians(0.3f);
        float turnUpDownPC = MathHelper.ToRadians(1f);
        float turnLeftRightXBOX = MathHelper.ToRadians(0.9f);
        float turnUpDownXBOX = MathHelper.ToRadians(1f);

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new human player profile
        /// </summary>
        /// <param name="playerId"></param>
        /// <param name="playerName"></param>
        /// <param name="isSurvivor"></param>
        /// <param name="playerPosition"></param>
        /// <param name="playerDirection"></param>
        /// <param name="viewPort"></param>
        /// <param name="content"></param>
        public HumanPlayerProfile(byte playerId, string playerName, bool isSurvivor, Vector3 playerPosition, Vector3 playerDirection, 
            Viewport viewPort, ContentManager content, GraphicsDevice graphicsDevice, Gamer gamer)
            : base(playerId, playerName, isSurvivor, playerPosition, playerDirection)
        {
            // Set values for instance variables
            this.viewPort = viewPort;
            this.camera = new Camera(playerPosition, playerDirection);
            this.lookUpDown = Vector3.Zero;
            this.cubeSelectionMenu = new CubeSelectionMenu(content);

            // Create the tools for the player
            this.tools = new Tool[3];
            this.tools[0] = new Hammer(content.Load<Model>("Tools/Hammer/hammer"), new PistolShotEffect(content, graphicsDevice));
            this.tools[1] = new Pistol(content.Load<Model>("Tools/Pistol/berreta89"), new PistolShotEffect(content, graphicsDevice));
            this.tools[2] = new AssaultRifle(content.Load<Model>("Tools/AssaultRifle/weapon"), new AssaultRifleShotEffect(content, graphicsDevice));

            // Create the avatar model
            this.playerModel = new AvatarModel(content, graphicsDevice);
            this.playerModel.LoadAvatar(gamer);

#if (!XBOX360)
            // Set the mouse position
            SetMousePosition();
#endif

        } // End of the constructor

        #endregion

        #region Player input

        public void HandlePlayerInput(InputState input, PlayerIndex playerIndex, BlockMap3D blockMap3D, CubeEngine cubeEngine, 
            AudioManager audioManager)
        {

            // Look up inputs for the specified player profile.
            KeyboardState currentKeyboardState = input.CurrentKeyboardStates[(int)playerIndex];
            KeyboardState previousKeyboardState = input.LastKeyboardStates[(int)playerIndex];
            GamePadState currentGamePadState = input.CurrentGamePadStates[(int)playerIndex];
            GamePadState previousGamePadState = input.LastGamePadStates[(int)playerIndex];
            MouseState mouseState = Mouse.GetState();
            
            // Set the previous player position.
            this.previousPlayerPosition = this.playerPosition;

            // What animation should we play, 0: stand, 4: walk, 5: jump, 9: shoot
            byte animationState = 0;

            // Handle cube menu input
            this.cubeSelectionMenu.HandleMenuInput(currentKeyboardState, previousKeyboardState,
                currentGamePadState, previousGamePadState, cubeEngine);

            // Do not handle any more input if the big cube menu is visible
            if (this.cubeSelectionMenu.isBigCubeMenuVisible == true)
                return;

            // Update cube instances and aiming collision
            if(blockMap3D != null)
                cubeEngine.UpdateCubeInstances(blockMap3D, this);

            // Move in different directions
            if (currentGamePadState.ThumbSticks.Left.Y > 0.25f || currentKeyboardState.IsKeyDown(Keys.W))
            {
                // Forward
                if(this.freeFlyingCamera == true)
                    this.playerPosition += (this.playerDirection + this.lookUpDown) * this.playerSpeed;
                else
                    this.playerPosition += this.playerDirection * this.playerSpeed;

                // Set the animation state
                animationState = 4;
            }
            if (currentGamePadState.ThumbSticks.Left.Y < -0.25f || currentKeyboardState.IsKeyDown(Keys.S))
            {
                // Back
                if (this.freeFlyingCamera == true)
                    this.playerPosition -= (this.playerDirection + this.lookUpDown) * this.playerSpeed;
                else
                    this.playerPosition -= this.playerDirection * this.playerSpeed;

                // Set the animation state
                animationState = 4;
            }
            if (currentGamePadState.ThumbSticks.Left.X < -0.25f || currentKeyboardState.IsKeyDown(Keys.A))
            {
                // Left
                Vector3 cameraDirection = Vector3.Cross(this.camera.cameraUp, this.playerDirection);
                if(this.freeFlyingCamera == true)
                    this.playerPosition += (cameraDirection + this.lookUpDown) * this.playerSpeed;
                else
                    this.playerPosition += cameraDirection * this.playerSpeed;

                // Set the animation state
                animationState = 4;

            }
            if (currentGamePadState.ThumbSticks.Left.X > 0.25f || currentKeyboardState.IsKeyDown(Keys.D))
            {
                // Right
                Vector3 cameraDirection = Vector3.Cross(this.camera.cameraUp, this.playerDirection);
                if(this.freeFlyingCamera == true)
                    this.playerPosition -= (cameraDirection + this.lookUpDown) * this.playerSpeed;
                else
                    this.playerPosition -= cameraDirection * this.playerSpeed;

                // Set the animation state
                animationState = 4;
            }

            // Change the tool
            if (currentGamePadState.IsButtonDown(Buttons.Y) && previousGamePadState.IsButtonUp(Buttons.Y) || currentKeyboardState.IsKeyDown(Keys.T) && previousKeyboardState.IsKeyUp(Keys.T))
            {
                // Calculate the current number.
                this.currentTool = (byte)((this.currentTool + 1) % this.tools.Length);
            }

            // Variable for rotation amount and up down amount, should be set by mouse or gamepad
            float rotationAmount = 0;
            float upDownAmount = 0;

#if (!XBOX360)
            // Look to the left and to the right (yaw)
            rotationAmount = (-this.turnLeftRightPC) * (Mouse.GetState().X - this.previousMouseState.X);

            // Look up and down (pitch)
            upDownAmount = (this.turnUpDownPC) * (Mouse.GetState().Y - this.previousMouseState.Y);
#endif
#if XBOX360
            // Gamepad input (look to the left and the right.
            rotationAmount = (-this.turnLeftRightXBOX) * (currentGamePadState.ThumbSticks.Right.X);

            // Gamepad input (look up and down).
            upDownAmount = (-this.turnUpDownXBOX) * (currentGamePadState.ThumbSticks.Right.Y);      
#endif

            // Set the up down amount
            float lookUpDownAmount = this.lookUpDown.Y;
            lookUpDownAmount -= upDownAmount;
            // If the Y value for look up and down is less then 90 or greater than -90
            if (lookUpDownAmount > MathHelper.ToRadians(90))
                this.lookUpDown.Y = MathHelper.ToRadians(90);
            else if (lookUpDownAmount < MathHelper.ToRadians(-90))
                this.lookUpDown.Y = MathHelper.ToRadians(-90);
            else
                this.lookUpDown.Y = lookUpDownAmount;

            // Set player direction and up down position.
            this.playerDirection = Vector3.Transform(this.playerDirection, Matrix.CreateFromAxisAngle(this.camera.cameraUp, rotationAmount));

            // *******************************************************************************************************
            // Check for a jump
            // *******************************************************************************************************
            if (currentGamePadState.IsButtonDown(Buttons.A) && previousGamePadState.IsButtonUp(Buttons.A) || 
                currentKeyboardState.IsKeyDown(Keys.Space) && previousKeyboardState.IsKeyUp(Keys.Space))
            {
                // Only jump if we not are jumping already
                if (this.jumpHeight == 0)
                    this.isJumping = true;

            }

            // Bring the player up if he is jumping
            if (this.isJumping == true)
            {
                if (this.jumpHeight < this.maxJumpHeight)
                {
                    // Make the player go up
                    this.jumpHeight += this.acceleration;
                    this.playerPosition.Y += this.acceleration;
                }
                else
                {
                    this.isJumping = false;
                }

                // Set the animation state
                animationState = 5;
            }
            // *******************************************************************************************************
            // End of the check for a jump section
            // *******************************************************************************************************

            if (this.freeFlyingCamera == false)
            {
                // Check if the player is outside of the map.
                CheckIfPlayerIsOutsideTheMap(blockMap3D);

                // Check for a player collision with a cube
                PlayerCollisionWithCube(blockMap3D, this.camera.cameraHeightOverPlayerFeets);
            }

            // Check for a ray collision with a cube
            AimingCollisionWithCube(blockMap3D);
            
            // ********************************************************************************************************
            // Add a block
            // ********************************************************************************************************
            if (currentGamePadState.IsButtonDown(Buttons.RightTrigger) && previousGamePadState.IsButtonUp(Buttons.RightTrigger) ||
                mouseState.LeftButton == ButtonState.Pressed && previousMouseState.LeftButton == ButtonState.Released)
            {
                if (this.currentTool == 0)
                {
                    // Add the block
                    blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0] + (int)this.addBlockVector.X, this.cubeSelectionMenu.selectedBlock[1] + (int)this.addBlockVector.Y, this.cubeSelectionMenu.selectedBlock[2] + (int)this.addBlockVector.Z] = this.cubeSelectionMenu.selectedBlockType;

                    // Play the add block sound
                    audioManager.Play3DSound("addBlockSound", false, this);
                }
            }

            // ********************************************************************************************************
            // Check if the player is shooting
            // ********************************************************************************************************
            if (currentGamePadState.IsButtonDown(Buttons.RightTrigger) || mouseState.LeftButton == ButtonState.Pressed)
            {
                if (this.currentTool != 0)
                    this.isShooting = true;
                else
                    this.isShooting = false;
            }
            else
            {
                this.isShooting = false;
            }

            // ********************************************************************************************************
            // Delete a block
            // ********************************************************************************************************
            if (currentGamePadState.IsButtonDown(Buttons.LeftTrigger) && previousGamePadState.IsButtonUp(Buttons.LeftTrigger) || 
                mouseState.RightButton == ButtonState.Pressed && previousMouseState.RightButton == ButtonState.Released)
            {
                if (this.currentTool == 0)
                {
                    // Get the block
                    Int16 selectedBlock = blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2]];

                    // Delete the block if it is different from zero.
                    if (selectedBlock > 0)
                        blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2]] = -1;

                    // Play the remove block sound
                    audioManager.Play3DSound("removeBlockSound", false, this);
                }
            }

            // Set the animation to play
            if (this.isShooting == true)
                this.playerModel.SetAnimation(9);
            else
                this.playerModel.SetAnimation(animationState);

            // Update the camera
            this.camera.Update(this.playerPosition, this.playerDirection, this.lookUpDown + this.playerDirection, this.viewPort);

            // Update player model rotation
            UpdatePlayerRotation(rotationAmount);

            // Update the mouse position
#if (!XBOX360)
            SetMousePosition();
#endif

        } // End of the HandlePlayerInput method

        /// <summary>
        /// Set the mouse position to the middle of the screen and set the previous mouse state.
        /// </summary>
        private void SetMousePosition()
        {
            // Reset the mouse to the middle of the screen
            Mouse.SetPosition(this.viewPort.Width / 2, this.viewPort.Height / 2);

            // Reset previous mouse state
            this.previousMouseState = Mouse.GetState();

        } // End of the SetMousePosition method

        #endregion

        #region Collision detection

        /// <summary>
        /// Check for a ray collision with a cube based on the direction in which the user is pointing.
        /// </summary>
        public void AimingCollisionWithCube(BlockMap3D blockMap3D)
        {

            // Just return if there not is a collision.
            if (this.aimingCollisionDistanceCube == float.MaxValue)
                return;

            // Get the position for the collision
            Vector3 collisionPosition = this.camera.cameraPosition + ((this.playerDirection + this.lookUpDown) * this.aimingCollisionDistanceCube);

            // Create a bounding box for the selected cube
            this.addBlockVector = Vector3.Zero;
            this.addBlockVector.Y = 1;
            Int16 cubeSize = (Int16)(blockMap3D.cubeSize);
            Vector3 minCubeVector = new Vector3(this.cubeSelectionMenu.selectedBlock[0] * cubeSize, this.cubeSelectionMenu.selectedBlock[1] * cubeSize, this.cubeSelectionMenu.selectedBlock[2] * cubeSize);
            BoundingBox selectedBox = new BoundingBox(minCubeVector, new Vector3(minCubeVector.X + cubeSize, minCubeVector.Y + cubeSize, minCubeVector.Z + cubeSize));

            if (collisionPosition.X == selectedBox.Max.X)
            {
                // X++
                if (this.cubeSelectionMenu.selectedBlock[0] + 1 < blockMap3D.mapWidthX && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0] + 1, this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2]] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.X = 1;
                }
            }
            if (collisionPosition.X == selectedBox.Min.X)
            {
                // X--
                if (this.cubeSelectionMenu.selectedBlock[0] - 1 >= 0 && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0] - 1, this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2]] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.X = -1;
                }
            }
            if (collisionPosition.Z == selectedBox.Max.Z)
            {
                // Z++
                if (this.cubeSelectionMenu.selectedBlock[2] + 1 < blockMap3D.mapDepthZ && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2] + 1] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.Z = 1;
                }
            }
            if (collisionPosition.Z == selectedBox.Min.Z)
            {
                // Z--
                if (this.cubeSelectionMenu.selectedBlock[2] - 1 >= 0 && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1], this.cubeSelectionMenu.selectedBlock[2] - 1] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.Z = -1;
                }
            }
            if (collisionPosition.Y == selectedBox.Max.Y)
            {
                // Y++
                if (this.cubeSelectionMenu.selectedBlock[1] + 1 < blockMap3D.mapHeightY && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1] + 1, this.cubeSelectionMenu.selectedBlock[2]] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.Y = 1;
                }
            }
            if (collisionPosition.Y == selectedBox.Min.Y)
            {
                // Y--
                if (this.cubeSelectionMenu.selectedBlock[1] - 1 >= 0 && blockMap3D.cubes[this.cubeSelectionMenu.selectedBlock[0], this.cubeSelectionMenu.selectedBlock[1] - 1, this.cubeSelectionMenu.selectedBlock[2]] == -1)
                {
                    this.addBlockVector = Vector3.Zero;
                    this.addBlockVector.Y = -1;
                }
            }

        } // End of the AimingCollisionWithCube method

        #endregion

    } // End of the class

} // End of the namespace
