﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.AvatarModels;
using AvatarBlockWar.GameLogic;
using AvatarBlockWar.Tools;
using MapObjects;
using AudioEngine;

namespace AvatarBlockWar.PlayerProfiles
{
    
    /// <summary>
    /// This class represent a player profile, it implements the 
    /// IComparable and the IAudioEmitter interface
    /// </summary>
    public class PlayerProfile : IComparable<PlayerProfile>, IAudioEmitter
    {
        #region Variables

        // Player identification
        public byte playerId;
        public string playerName;
        public bool isSurvivor;

        // Player position and speed
        public Vector3 playerPosition;
        public Vector3 previousPlayerPosition;
        public Vector3 playerDirection;
        public float playerSpeed = 1f;

        // Player rotation
        public Matrix rotationMatrix;
        public Quaternion AvatarRotation;

        // Gravitation and jumping
        public float jumpHeight = 0f;
        public float acceleration = 1f;
        public bool isJumping = false;
        public float maxJumpHeight = 50f;

        // Player model (Avatar)
        public AvatarModel playerModel;

        // Tools
        public Tool[] tools;
        public byte currentTool = 0;

        // Shooting
        public bool isShooting = false;
        public Bullet bullet = null;
        public int shotDelayCounter = 0;

        // Collision distances
        public float bulletCollisionDistanceCube = float.MaxValue;
        public float aimingCollisionDistanceCube = float.MaxValue;
        
        // Health and status
        public byte health = 100;
        public bool isDead = false;
        public string killedByGamerTag;
        public int spawnCountDown = 10000;

        // Statistics
        public short kills = 0;
        public short deaths = 0;

        // Walk sound delay and index
        protected int walkDelay = 470;
        protected int walkCountdown = 0;
        protected int walkSoundIndex = 0;

        #endregion

        #region Audio properties

        /// <summary>
        /// Gets or sets the 3D position of the entity.
        /// </summary>
        private Vector3 position;
        public Vector3 Position
        {
            get { return position; }
            set { position = value; }
        }

        /// <summary>
        /// Gets or sets which way the entity is facing.
        /// </summary>
        private Vector3 forward;
        public Vector3 Forward
        {
            get { return forward; }
            set { forward = value; }
        }

        /// <summary>
        /// Gets or sets the orientation of this entity.
        /// </summary>
        private Vector3 up;
        public Vector3 Up
        {
            get { return up; }
            set { up = value; }
        }

        /// <summary>
        /// Gets or sets how fast this entity is moving.
        /// </summary>
        private Vector3 velocity;
        public Vector3 Velocity
        {
            get { return velocity; }
            protected set { velocity = value; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new player profile
        /// </summary>
        /// <param name="playerId"></param>
        /// <param name="playerName"></param>
        /// <param name="playerPosition"></param>
        /// <param name="playerDirection"></param>
        public PlayerProfile(byte playerId, string playerName, bool isSurvivor, Vector3 playerPosition, Vector3 playerDirection)
        {
            // Set values for instance variables
            this.playerId = playerId;
            this.playerName = playerName;
            this.isSurvivor = isSurvivor;
            this.playerPosition = playerPosition;
            this.playerDirection = playerDirection;

            // Set the rotation matrix
            this.rotationMatrix = Matrix.Identity;

        } // End of the constructor

        #endregion

        #region Update

        /// <summary>
        /// Update the rotation for the player model.
        /// </summary>
        /// <param name="turnAmount"></param>
        public void UpdatePlayerRotation(float turnAmount)
        {
            // Update the rotation matrix for the soldier
            this.rotationMatrix *= Matrix.CreateRotationY(turnAmount);

            // Update the rotation quaternion.
            this.AvatarRotation = Quaternion.CreateFromRotationMatrix(this.rotationMatrix);

        } // End of the UpdatePlayerRotation method

        /// <summary>
        /// Update the spawn time for a dead soldier
        /// </summary>
        /// <param name="gameTime"></param>
        public void UpdateSpawnTime(GameTime gameTime)
        {
            // Decrease the spawn time counter
            if (this.isDead == true)
            {
                this.spawnCountDown -= gameTime.ElapsedGameTime.Milliseconds;
            }

            // Check if we should set the player as alive.
            if (this.spawnCountDown <= 0)
            {
                // The player is alive and not wounded any more.
                this.isDead = false;
                this.health = 100;

                // Change the position
                this.playerPosition = StaticPlayerProfileMethods.getRandomStartPosition();

                // Reset the spawn countdown (10 seconds)
                this.spawnCountDown = 10000;
            }

        } // End of the UpdateSpawnTime method

        /// <summary>
        /// This method will update the audio emitter and play a walking sound
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="audioManager"></param>
        public void UpdateAudio(GameTime gameTime, AudioManager audioManager)
        {

            // Update audio emitter options
            this.Position = this.playerPosition;
            this.Forward = this.playerDirection;
            this.Up = Vector3.Up;
            this.Velocity = this.playerDirection * this.playerSpeed;

            // Play the walking sound.
            if ((int)this.playerModel.currentType == 4)
            {
                if (this.walkCountdown <= 0)
                {
                    // For variety, choose between (footstep0 or footstep1).
                    string soundName;
                    if (this.walkSoundIndex == 0)
                    {
                        soundName = "footstep" + this.walkSoundIndex++;
                    }
                    else
                    {
                        soundName = "footstep" + this.walkSoundIndex--;
                    }

                    // Play the sound.
                    audioManager.Play3DSound(soundName, false, this);

                    // Reset the walking counter.
                    this.walkCountdown = this.walkDelay;
                }
                else
                {
                    // Decrease the counter.
                    this.walkCountdown -= gameTime.ElapsedGameTime.Milliseconds;
                }
            }

        } // End of the UpdateAudio method

        #endregion

        #region Collision detection

        /// <summary>
        /// Check if a player is hit by a bullet.
        /// </summary>
        /// <param name="playerProfiles"></param>
        public void CheckIfPlayersIsHitByBullet(List<PlayerProfile> playerProfiles, GamePlayLogic gameLogic)
        {

            // Create a bounding box for the player
            Vector3 playerMinVector = new Vector3(this.playerPosition.X - 10, this.playerPosition.Y, this.playerPosition.Z - 10);
            Vector3 playerMaxVector = new Vector3(this.playerPosition.X + 10, this.playerPosition.Y + 50, this.playerPosition.Z + 10);
            BoundingBox playerBox = new BoundingBox(playerMinVector, playerMaxVector);

            // ***************************************************************************************
            // Check every bullet against the player
            // ***************************************************************************************
            foreach (PlayerProfile enemy in playerProfiles)
            {
                // Just continue if the enemy player profile is the same as the player profile.
                if (this.playerName == enemy.playerName)
                    continue;

                // Check if the bullet collides with the player.
                if (enemy.bullet != null && playerBox.Intersects(enemy.bullet.bulletRay) != null)
                {
                    // We have a hit if the collision distance is less than the collision distance with
                    // cube.
                    if (playerBox.Intersects(enemy.bullet.bulletRay) < enemy.bulletCollisionDistanceCube)
                    {
                        // Decrease the health
                        this.health -= enemy.tools[enemy.currentTool].healthDamage;

                        // Remove the enemy bullet
                        enemy.bullet = null;

                        // Check if the player is dead
                        if (this.health <= 0)
                        {
                            // Mark the player as dead.
                            this.isDead = true;
                            this.killedByGamerTag = enemy.playerName;

                            // Add statistics
                            enemy.kills++;
                            this.deaths++;

                            // Add the blood effect
                            Vector3 bloodStartPosition = this.playerPosition + new Vector3(0, 50, 0);
                            this.playerModel.bloodEffect.AddParticles(4, bloodStartPosition, Vector3.Zero);

                            // Play the death animation
                            this.playerModel.SetAnimation(8);

                            // Decrease the number of lives left
                            if (this.isSurvivor == true)
                                gameLogic.numberOfLives--;

                            break;
                        }
                    }
                }
            }

        } // End of the CheckIfPlayersIsHitByBullet method

        /// <summary>
        /// Check if the player is outside of the map and make adjustments accordingly
        /// </summary>
        public void CheckIfPlayerIsOutsideTheMap(BlockMap3D blockMap3D)
        {

            // Just return if a map not exists.
            if (blockMap3D == null)
                return;

            // Declare variables for the max positions
            int maxX = ((blockMap3D.mapWidthX - 1) * blockMap3D.cubeSize);
            int maxZ = ((blockMap3D.mapDepthZ - 1) * blockMap3D.cubeSize);

            if (this.playerPosition.X < 0)
                this.playerPosition.X = 0;
            if (this.playerPosition.X > maxX)
                this.playerPosition.X = maxX;
            if (this.playerPosition.Z < 0)
                this.playerPosition.Z = 0;
            if (this.playerPosition.Z > maxZ)
                this.playerPosition.Z = maxZ;

        } // End of the CheckIfPlayerIsOutsideTheMap method

        /// <summary>
        /// Check for a player collison with a cube
        /// </summary>
        public void PlayerCollisionWithCube(BlockMap3D blockMap3D, float heightOverPlayersFeet)
        {

            // Just return if a map not exists.
            if (blockMap3D == null)
                return;

            // Calculate where the player is.
            int xTile = (int)this.playerPosition.X / blockMap3D.cubeSize;
            int zTile = (int)this.playerPosition.Z / blockMap3D.cubeSize;
            int yTile = (int)this.playerPosition.Y / blockMap3D.cubeSize;

            if (xTile >= blockMap3D.mapWidthX)
                xTile = blockMap3D.mapWidthX - 1;
            if (zTile >= blockMap3D.mapDepthZ)
                zTile = blockMap3D.mapDepthZ - 1;
            if (yTile >= blockMap3D.mapHeightY)
                yTile = blockMap3D.mapHeightY - 1;

            // If there is no list of cubes, just get out from this method.
            //if (blockMap3D.cubes.Length < 1)
            //    return;

            // Create a bounding box for the player
            Vector3 playerMinVector = new Vector3(this.playerPosition.X - 10, this.playerPosition.Y, this.playerPosition.Z - 10);
            Vector3 playerMaxVector = new Vector3(this.playerPosition.X + 10, this.playerPosition.Y + heightOverPlayersFeet, this.playerPosition.Z + 10);
            BoundingBox playerBox = new BoundingBox(playerMinVector, playerMaxVector);

            // Check boxes in three height levels.
            int yTileCollision = yTile;
            bool playerCollision = false;
            for (int y = 0; y < 2; y++)
            {
                // Break out from the loop if there has been a collision
                if (playerCollision == true)
                    break;

                // Create a bounding box for the tile
                Vector3 minVector = new Vector3(xTile * blockMap3D.cubeSize, yTileCollision * blockMap3D.cubeSize, zTile * blockMap3D.cubeSize);
                BoundingBox tileBox = new BoundingBox(minVector, new Vector3(minVector.X + blockMap3D.cubeSize, minVector.Y + blockMap3D.cubeSize, minVector.Z + blockMap3D.cubeSize));

                // Test for a collision with a cube and respond to the collision
                if (blockMap3D.cubes[xTile, yTileCollision, zTile] != -1 && playerBox.Intersects(tileBox))
                {
                    // Calculate the smallest distance from the previous position
                    float distanceToMaxX = Math.Abs(this.previousPlayerPosition.X - tileBox.Max.X);
                    float distanceToMinX = Math.Abs(this.previousPlayerPosition.X - tileBox.Min.X);
                    float distanceToMaxZ = Math.Abs(this.previousPlayerPosition.Z - tileBox.Max.Z);
                    float distanceToMinZ = Math.Abs(this.previousPlayerPosition.Z - tileBox.Min.Z);

                    // Adjust the player position
                    if (distanceToMaxX < distanceToMinX && distanceToMaxX < distanceToMaxZ && distanceToMaxX < distanceToMinZ)
                    {
                        if (blockMap3D.cubes[xTile + 1, yTileCollision, zTile] == -1)
                        {
                            // X++
                            this.playerPosition.X = this.previousPlayerPosition.X;
                            playerCollision = true;
                        }
                        else
                        {
                            if (distanceToMaxZ < distanceToMinZ)
                            {
                                // Z++
                                this.playerPosition.Z = this.previousPlayerPosition.Z;

                                // X++
                                if (blockMap3D.cubes[xTile, yTileCollision, zTile + 1] != -1)
                                    this.playerPosition.X = this.previousPlayerPosition.X;

                                playerCollision = true;
                            }
                            else if (distanceToMinZ < distanceToMaxZ)
                            {
                                // Z--
                                this.playerPosition.Z = this.previousPlayerPosition.Z;

                                // X++
                                if (blockMap3D.cubes[xTile, yTileCollision, zTile - 1] != -1)
                                    this.playerPosition.X = this.previousPlayerPosition.X;

                                playerCollision = true;
                            }
                        }
                    }
                    if (distanceToMinX < distanceToMaxX && distanceToMinX < distanceToMaxZ && distanceToMinX < distanceToMinZ)
                    {
                        if (blockMap3D.cubes[xTile - 1, yTileCollision, zTile] == -1)
                        {
                            // X--
                            this.playerPosition.X = this.previousPlayerPosition.X;
                            playerCollision = true;
                        }
                        else
                        {
                            if (distanceToMaxZ < distanceToMinZ)
                            {
                                // Z++
                                this.playerPosition.Z = this.previousPlayerPosition.Z;

                                // X--
                                if (blockMap3D.cubes[xTile, yTileCollision, zTile + 1] != -1)
                                    this.playerPosition.X = this.previousPlayerPosition.X;

                                playerCollision = true;
                            }
                            else if (distanceToMinZ < distanceToMaxZ)
                            {
                                // Z--
                                this.playerPosition.Z = this.previousPlayerPosition.Z;

                                // X--
                                if (blockMap3D.cubes[xTile, yTileCollision, zTile - 1] != -1)
                                    this.playerPosition.X = this.previousPlayerPosition.X;

                                playerCollision = true;
                            }
                        }
                    }
                    if (distanceToMaxZ < distanceToMaxX && distanceToMaxZ < distanceToMinX && distanceToMaxZ < distanceToMinZ)
                    {
                        if (blockMap3D.cubes[xTile, yTileCollision, zTile + 1] == -1)
                        {
                            // Z++
                            this.playerPosition.Z = this.previousPlayerPosition.Z;
                            playerCollision = true;
                        }
                        else
                        {
                            if (distanceToMaxX < distanceToMinX)
                            {
                                // X++
                                this.playerPosition.X = this.previousPlayerPosition.X;

                                // Z++
                                if (blockMap3D.cubes[xTile + 1, yTileCollision, zTile] != -1)
                                    this.playerPosition.Z = this.previousPlayerPosition.Z;

                                playerCollision = true;
                            }
                            else if (distanceToMinX < distanceToMaxX)
                            {
                                // X--
                                this.playerPosition.X = this.previousPlayerPosition.X;

                                // Z++
                                if (blockMap3D.cubes[xTile - 1, yTileCollision, zTile] != -1)
                                    this.playerPosition.Z = this.previousPlayerPosition.Z;

                                playerCollision = true;
                            }
                        }
                    }
                    if (distanceToMinZ < distanceToMaxX && distanceToMinZ < distanceToMinX && distanceToMinZ < distanceToMaxZ)
                    {
                        if (blockMap3D.cubes[xTile, yTileCollision, zTile - 1] == -1)
                        {
                            // Z--
                            this.playerPosition.Z = this.previousPlayerPosition.Z;
                            playerCollision = true;
                        }
                        else
                        {
                            if (distanceToMaxX < distanceToMinX)
                            {
                                // X++
                                this.playerPosition.X = this.previousPlayerPosition.X;

                                // Z--
                                if (blockMap3D.cubes[xTile + 1, yTileCollision, zTile] != -1)
                                    this.playerPosition.Z = this.previousPlayerPosition.Z;

                                playerCollision = true;
                            }
                            else if (distanceToMinX < distanceToMaxX)
                            {
                                // X--
                                this.playerPosition.X = this.previousPlayerPosition.X;

                                // Z--
                                if (blockMap3D.cubes[xTile - 1, yTileCollision, zTile] != -1)
                                    this.playerPosition.Z = this.previousPlayerPosition.Z;

                                playerCollision = true;
                            }
                        }

                    } // End of the adjustment of the player position part.

                } // End of if (blockMap3D.cubes[xTile, yTileCollision, zTile] != -1 && cameraBox.Intersects(tileBox))

                // Add one to yTileCollision
                if (yTileCollision < (blockMap3D.mapHeightY - 1))
                    yTileCollision++;

            } // End of the for (int y = 0; y < 3; y++) loop

            // Return if we have a free flying camera.
            float playerPositionY = this.playerPosition.Y;

            // Bring the player down if he not is jumping, apply gravity
            if (this.isJumping == false)
            {
                // Bring the player down....
                playerPositionY -= this.acceleration;

                // ...but not below zero
                if (playerPositionY < 0)
                    playerPositionY = 0;
            }

            // Calculate where the player is again.
            xTile = (int)this.playerPosition.X / blockMap3D.cubeSize;
            zTile = (int)this.playerPosition.Z / blockMap3D.cubeSize;
            yTile = (int)playerPositionY / blockMap3D.cubeSize;

            if (xTile >= blockMap3D.mapWidthX)
                xTile = blockMap3D.mapWidthX - 1;
            if (zTile >= blockMap3D.mapDepthZ)
                zTile = blockMap3D.mapDepthZ - 1;
            if (yTile >= blockMap3D.mapHeightY)
                yTile = blockMap3D.mapHeightY - 1;

            // Check for a player collision with a cube in Y position
            if (blockMap3D.cubes[xTile, yTile, zTile] != -1)
            {
                this.playerPosition.Y = (yTile * blockMap3D.cubeSize) + blockMap3D.cubeSize;
                this.jumpHeight = 0f;
            }
            else
            {
                // Bring the player down with gravity
                this.playerPosition.Y = playerPositionY;           
            }

        } // End of the PlayerCollisionWithCube method.

        #endregion

        #region Methods

        /// <summary>
        /// Compare one player against another on number of kills.
        /// </summary>
        /// <param name="other"></param>
        /// <returns>An int -1, 0 or 1</returns>
        public virtual int CompareTo(PlayerProfile other)
        {
            // Sort in descending order.
            return other.kills.CompareTo(this.kills);
        }

        #endregion

    } // End of the class

} // End of the namespace
