﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using AvatarBlockWar.Screens.MenuScreens;
using AvatarBlockWar.PlayerProfiles;
using ScreenManagement;
using Networking;

namespace AvatarBlockWar.Screens.GameScreens
{
    /// <summary>
    /// This class represent a end game screen
    /// </summary>
    public class EndGameScreen : MenuScreen
    {
        #region Variables

        // A reference to the network session
        NetworkSession networkSession;

        // Game statistics
        private string winningTeam;
        private List<PlayerProfile> players;

        // Constant x and y offsets
        const int X_OFFSET = 128;
        const int Y_OFFSET = 72;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new end game screen
        /// </summary>
        /// <param name="networkSession"></param>
        public EndGameScreen(NetworkSession networkSession, string winningTeam, List<PlayerProfile> playerProfiles)
            : base(Resources.EndOfGame)
        {
            // Set instance variables
            this.networkSession = networkSession;
            this.winningTeam = winningTeam;
            this.players = playerProfiles;

            // Add menu entries
            if (networkSession == null)
            {
                // If this is a single player game, add the Quit menu entry.
                MenuEntry quitGameMenuEntry = new MenuEntry(Resources.QuitGame);
                quitGameMenuEntry.Selected += QuitGameMenuEntrySelected;
                menuEntries.Add(quitGameMenuEntry);
            }
            else
            {
                // If we are hosting a network game, add the Return to Lobby menu entry.
                if (networkSession.IsHost)
                {
                    MenuEntry lobbyMenuEntry = new MenuEntry(Resources.ReturnToLobby);
                    lobbyMenuEntry.Selected += ReturnToLobbyMenuEntrySelected;
                    menuEntries.Add(lobbyMenuEntry);
                }

                // Add the End/Leave Session menu entry.
                string leaveEntryText = networkSession.IsHost ? Resources.EndSession :
                                                                Resources.LeaveSession;

                MenuEntry leaveSessionMenuEntry = new MenuEntry(leaveEntryText);
                leaveSessionMenuEntry.Selected += LeaveSessionMenuEntrySelected;
                menuEntries.Add(leaveSessionMenuEntry);
            }

            // Sort the player list on number of kills
            this.players.Sort();

        } // End of the constructor

        #endregion

        #region Draw

        /// <summary>
        /// Draw on the end game screen.
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Draw(GameTime gameTime)
        {

            base.Draw(gameTime);

            // Create reference variables.
            GraphicsDevice graphicsDevice = ScreenManager.GraphicsDevice;
            SpriteBatch spriteBatch = ScreenManager.spriteBatch;
            SpriteFont font = ScreenManager.font;

            // Start draw in 2D
            spriteBatch.Begin();

            // Properties for text on the end game screen
            String winningTeamString = this.winningTeam + " won the game!";
            float textScale = 0.7f;
            Color textColor = Color.White * TransitionAlpha;

            // Draw the winning team.
            spriteBatch.DrawString(font, winningTeamString, new Vector2(X_OFFSET, Y_OFFSET), textColor, 0,
                                   Vector2.Zero, 1.5f, SpriteEffects.None, 0);

            // A offset for each name to print.
            int secondColumnXoffset = 300;
            int offsetFirstColumn = 25;
            int offsetSecondColumn = 25;

            // Draw each player in the list
            for (int i = 0; i < players.Count; i++)
            {

                // Create the player string.
                String playerString = (i + 1) + ". " + players[i].playerName + " " + players[i].kills;

                // Draw 8 players on each side of the screen.
                if (i < 8)
                {
                    spriteBatch.DrawString(font, playerString, new Vector2(X_OFFSET, 300 + offsetFirstColumn), textColor, 0,
                                       Vector2.Zero, textScale, SpriteEffects.None, 0);

                    // Add to the offset
                    offsetFirstColumn += 25;
                }
                else
                {
                    spriteBatch.DrawString(font, playerString, new Vector2(X_OFFSET + secondColumnXoffset, 300 + offsetSecondColumn), textColor, 0,
                                       Vector2.Zero, textScale, SpriteEffects.None, 0);

                    // Add to the offset
                    offsetSecondColumn += 25;
                }
            }

            spriteBatch.End();

        } // End of the Draw method

        #endregion

        #region EventHandlers

        /// <summary>
        /// Event handler for when the Quit Game menu entry is selected.
        /// </summary>
        private void QuitGameMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            MessageBoxScreen confirmQuitMessageBox = new MessageBoxScreen(Resources.ConfirmQuitGame);

            confirmQuitMessageBox.Accepted += ConfirmQuitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmQuitMessageBox, ControllingPlayer);
        }

        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure
        /// you want to quit" message box. This uses the loading screen to
        /// transition from the game back to the main menu screen.
        /// </summary>
        private void ConfirmQuitMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {
            LoadingScreen.Load(ScreenManager, false, null, new BackgroundScreen(), new MainMenuScreen());
        }

        /// <summary>
        /// Event handler for when the Return to Lobby menu entry is selected.
        /// </summary>
        private void ReturnToLobbyMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            if (networkSession.SessionState == NetworkSessionState.Playing)
            {
                networkSession.EndGame();
            }
        }

        /// <summary>
        /// Event handler for when the End/Leave Session menu entry is selected.
        /// </summary>
        private void LeaveSessionMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            NetworkSessionComponent.LeaveSession(ScreenManager, e.PlayerIndex);
        }

        #endregion

    } // End of class

} // End of namespace
