
using System;
using System.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using AvatarBlockWar.Screens.MenuScreens;
using AvatarBlockWar.Screens.LobbyScreens;
using AvatarBlockWar.Screens.NetworkScreens;
using AvatarBlockWar.GameLogic;
using MapObjects;
using CubeEngines;
using ScreenManagement;
using Networking;

namespace AvatarBlockWar.Screens.GameScreens
{
    /// <summary>
    /// This screen represent the actual gameplay
    /// </summary>
    public class GameplayScreen : GameScreen
    {
        #region Variables

        // A reference to the current network session
        NetworkSession networkSession;

        // A content manager
        ContentManager content;
  
        // The current game mode, 0: Build map, 1: Local network game, 2: Wide area network
        byte gameMode;
        string mapName;
        GamePlayLogic gameLogic;

        // Transparency for fading screens when pausing.
        float pauseAlpha;

        #endregion

        #region Properties

        /// <summary>
        /// The logic for deciding whether the game is paused depends on whether
        /// this is a networked or a local game.
        /// </summary>
        new bool IsActive
        {
            get
            {
                if (this.gameMode == 2)
                {
                    // Pause behavior for networked games.
                    return !IsExiting;       
                }
                else
                {
                    // Pause behavior for local games.
                    return base.IsActive;
                }
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new gameplay screen.
        /// </summary>
        public GameplayScreen(NetworkSession networkSession, byte gameMode, string mapName)
        {
            // Set instance variables
            this.networkSession = networkSession;
            this.gameMode = gameMode;
            this.mapName = mapName;

            TransitionOnTime = TimeSpan.FromSeconds(1.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);

        } // End of the constructor

        #endregion

        #region Initialization

        /// <summary>
        /// Load graphics content for the game.
        /// </summary>
        public override void LoadContent()
        {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

            // Set the gameplay logic
            switch (this.gameMode)
            {
                case 0:
                    this.gameLogic = new BuildMapLogic(content, this.networkSession, ScreenManager.GraphicsDevice, ControllingPlayer.Value, this.mapName);
                    break;
                case 1:
                    this.gameLogic = new LocalGameLogic(content, this.networkSession, ScreenManager.GraphicsDevice, this.mapName);
                    break;
            }

            // Reset the elapsed time
            ScreenManager.Game.ResetElapsedTime();
        }

        /// <summary>
        /// Unload graphics content used by the game.
        /// </summary>
        public override void UnloadContent()
        {
            // Dispose the audio manager
            this.gameLogic.DisposeAudio();

            content.Unload();

        } // End of the UnloadContent method

        #endregion

        #region Update and Draw

        /// <summary>
        /// Updates the state of the game.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus, bool coveredByOtherScreen)
        {
            base.Update(gameTime, otherScreenHasFocus, false);

            // Gradually fade in or out depending on whether we are covered by the pause screen.
            if (coveredByOtherScreen)
                pauseAlpha = Math.Min(pauseAlpha + 1f / 32, 1);
            else
                pauseAlpha = Math.Max(pauseAlpha - 1f / 32, 0);

            if (IsActive)
            {
                // Update the game logic
                this.gameLogic.Update(gameTime);
            }

            // If we are in a network game, check if we should return to the lobby.
            if ((networkSession != null) && !IsExiting)
            {
                if (networkSession.SessionState == NetworkSessionState.Lobby)
                {
                    if(this.gameMode == 1)
                        LoadingScreen.Load(ScreenManager, true, null, new BackgroundScreen(), new LocalGameLobbyScreen(networkSession));
                }
            }

        } // End of the Update method


        /// <summary>
        /// Lets the game respond to player input. Unlike the Update method,
        /// this will only be called when the gameplay screen is active.
        /// </summary>
        public override void HandleInput(InputState input)
        {
            if (input == null)
                throw new ArgumentNullException("input");

            if (ControllingPlayer.HasValue)
            {
                // In single player games, handle input for the controlling player.
                HandlePlayerInput(input, null,  ControllingPlayer.Value);
            }
            else if (networkSession != null)
            {
                // In network game modes, handle input for all the
                // local players who are participating in the session.
                foreach (LocalNetworkGamer gamer in networkSession.LocalGamers)
                {
                    if (!HandlePlayerInput(input, gamer, gamer.SignedInGamer.PlayerIndex))
                        break;
                }
            }

        } // End of the HandleInput method


        /// <summary>
        /// Handles input for the specified player. In local game modes, this is called
        /// just once for the controlling player. In network modes, it can be called
        /// more than once if there are multiple profiles playing on the local machine.
        /// Returns true if we should continue to handle input for subsequent players,
        /// or false if this player has paused the game.
        /// </summary>
        bool HandlePlayerInput(InputState input, LocalNetworkGamer gamer, PlayerIndex playerIndex)
        {

            // Look up inputs for the specified player profile.
            GamePadState gamePadState = input.CurrentGamePadStates[(int)playerIndex];

            // The game pauses either if the user presses the pause button, or if
            // they unplug the active gamepad.
            bool gamePadDisconnected = !gamePadState.IsConnected && input.GamePadWasConnected[(int)playerIndex];

            // Add a pause screen if the game is pausing.
            if (input.IsPauseGame(playerIndex) || gamePadDisconnected)
            {
                ScreenManager.AddScreen(new PauseMenuScreen(networkSession, this.gameLogic), playerIndex);
                return false;
            }

            // Handle player input
            this.gameLogic.HandlePlayerInput(gamer, input, playerIndex);

            // Add the end game screen if the game has ended.
            if (this.gameLogic.gameHasEnded == true)
            {
                // Add the end game screen
                ScreenManager.AddScreen(new EndGameScreen(networkSession, this.gameLogic.winningTeam, this.gameLogic.playerProfiles), null);
                return false;
            }

            return true;

        } // End of the HandlePlayerInput method


        /// <summary>
        /// Draws the gameplay screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
        {
            // Clear out the graphics device
            ScreenManager.GraphicsDevice.Clear(ClearOptions.Target, Color.Black, 0, 0);

            // Draw the game logic
            this.gameLogic.Draw(gameTime, ScreenManager);

            // If the game is transitioning on or off, fade it out to black.
            if (TransitionPosition > 0 || pauseAlpha > 0)
            {
                float alpha = MathHelper.Lerp(1f - TransitionAlpha, 1f, pauseAlpha / 2);

                ScreenManager.FadeBackBufferToBlack(alpha);
            }

        } // End of the Draw method

        #endregion

    } // End of the class

} // End of the namespace
