#region File Description
//-----------------------------------------------------------------------------
// PauseMenuScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Net;
using ScreenManagement;
using AvatarBlockWar.Screens.MenuScreens;
using AvatarBlockWar.DataStorage;
using AvatarBlockWar.GameLogic;
using Networking;

namespace AvatarBlockWar.Screens.GameScreens
{
    /// <summary>
    /// The pause menu comes up over the top of the game,
    /// giving the player options to resume or quit.
    /// </summary>
    class PauseMenuScreen : MenuScreen
    {
        #region Variables

        NetworkSession networkSession;
        GamePlayLogic gameLogic;
        
        #endregion

        #region Initialization

        /// <summary>
        /// Create a new pause menu screen.
        /// </summary>
        public PauseMenuScreen(NetworkSession networkSession, GamePlayLogic gameLogic)
            : base(Resources.Paused)
        {
            this.networkSession = networkSession;
            this.gameLogic = gameLogic;

            // Add the Resume Game menu entry.
            MenuEntry resumeGameMenuEntry = new MenuEntry(Resources.ResumeGame);
            resumeGameMenuEntry.Selected += OnCancel;
            menuEntries.Add(resumeGameMenuEntry);

            // Add the controller settings menu entry.
            MenuEntry controllerSettingsMenuEntry = new MenuEntry(Resources.ControllerSettings);
            controllerSettingsMenuEntry.Selected += ControllerSettingsMenuEntrySelected;
            menuEntries.Add(controllerSettingsMenuEntry);

            if (networkSession == null)
            {
                // If this is a single player game, add the Save map menu entry.
                MenuEntry saveMapMenuEntry = new MenuEntry("Save map");
                saveMapMenuEntry.Selected += SaveMapMenuEntrySelected;
                menuEntries.Add(saveMapMenuEntry);

                // If this is a single player game, add the Quit menu entry.
                MenuEntry quitGameMenuEntry = new MenuEntry(Resources.QuitGame);
                quitGameMenuEntry.Selected += QuitGameMenuEntrySelected;
                menuEntries.Add(quitGameMenuEntry);
            }
            else
            {
                // If we are hosting a network game, add the Return to Lobby menu entry.
                if (networkSession.IsHost)
                {
                    MenuEntry lobbyMenuEntry = new MenuEntry(Resources.ReturnToLobby);
                    lobbyMenuEntry.Selected += ReturnToLobbyMenuEntrySelected;
                    menuEntries.Add(lobbyMenuEntry);
                }

                // Add the End/Leave Session menu entry.
                string leaveEntryText = networkSession.IsHost ? Resources.EndSession :
                                                                Resources.LeaveSession;

                MenuEntry leaveSessionMenuEntry = new MenuEntry(leaveEntryText);
                leaveSessionMenuEntry.Selected += LeaveSessionMenuEntrySelected;
                menuEntries.Add(leaveSessionMenuEntry);
            }
        }


        #endregion

        #region Handle Input

        /// <summary>
        /// Event handler for when the Controller Settings menu entry is selected.
        /// </summary>
        private void ControllerSettingsMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Add a game pad control screen.
            //ScreenManager.AddScreen(new ControllerSettingsScreen(e.PlayerIndex), e.PlayerIndex);
        }

        /// <summary>
        /// Event handler for when the Quit Game menu entry is selected.
        /// </summary>
        private void QuitGameMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            MessageBoxScreen confirmQuitMessageBox =
                                    new MessageBoxScreen(Resources.ConfirmQuitGame);

            confirmQuitMessageBox.Accepted += ConfirmQuitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmQuitMessageBox, ControllingPlayer);
        }

        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure
        /// you want to quit" message box. This uses the loading screen to
        /// transition from the game back to the main menu screen.
        /// </summary>
        private void ConfirmQuitMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {

            LoadingScreen.Load(ScreenManager, false, null, new BackgroundScreen(),
                                                           new MainMenuScreen());
        }

        /// <summary>
        /// Event handler for when save map menu entry is selected.
        /// </summary>
        private void SaveMapMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Save the map
            MapDataStorage.BeginSaveMapToFile(this.gameLogic.currentBlockMap, this.gameLogic.mapName, ScreenManager, Program.commonStorageDevice);
        }

        /// <summary>
        /// Event handler for when the Return to Lobby menu entry is selected.
        /// </summary>
        private void ReturnToLobbyMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            if (networkSession.SessionState == NetworkSessionState.Playing)
            {
                networkSession.EndGame();
            }
        }

        /// <summary>
        /// Event handler for when the End/Leave Session menu entry is selected.
        /// </summary>
        private void LeaveSessionMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            NetworkSessionComponent.LeaveSession(ScreenManager, e.PlayerIndex);
        }

        #endregion

    } // End of the class

} // End of the namespace
