﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using AvatarBlockWar.DataStorage;
using ScreenManagement;

namespace AvatarBlockWar.Screens.LobbyScreens
{
    /// <summary>
    /// This class represent a screen for game settings.
    /// </summary>
    public class GameSettingsScreen : MenuScreen
    {
        #region Variables

        // Menu entries
        private MenuEntry mapMenuEntry;
        private MenuEntry numberOfLivesMenuEntry;
        private MenuEntry gameTimeMenuEntry;
        private MenuEntry saveAndExitMenuEntry;

        // Variables for map names
        private List<string> maps;
        public int currentMap = 0;

        // Variables for number of lives and game time
        private int currentNumberOfLives = 20;
        private int currentGameTime = 5; // Minutes

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new game settings screen.
        /// </summary>
        public GameSettingsScreen()
            : base(Resources.Settings)
        {
            // Get all map names
            string[] mapNames = MapDataStorage.GetMapNames(Program.commonStorageDevice);
            
            // Create the list of map names
            this.maps = new List<string>(mapNames.Length);
            for (int map = 0; map < mapNames.Length; map++)
            {
                // Add the map name without the extension to the list of map names.
                this.maps.Add(mapNames[map].Split('.')[0]);
            }

            // Get the current game settings
            this.currentMap = Program.selectedMap;
            this.currentNumberOfLives = Program.numberOfLives;
            this.currentGameTime = (Program.totalGameTimeMilliseconds / 60000);

            // Create menu entries
            this.mapMenuEntry = new MenuEntry("");
            this.numberOfLivesMenuEntry = new MenuEntry("");
            this.gameTimeMenuEntry = new MenuEntry("");
            this.saveAndExitMenuEntry = new MenuEntry("Save and exit");

            // Set the mapMenuEntry text.
            SetMenuEntryText();

            // Hook up menu event handlers
            this.mapMenuEntry.Selected += MapMenuEntrySelected;
            this.numberOfLivesMenuEntry.Selected += NumberOfLivesMenuEntrySelected;
            this.gameTimeMenuEntry.Selected += GameTimeMenuEntrySelected;
            this.saveAndExitMenuEntry.Selected += SaveAndExitMenuEntrySelected;

            // Add menu entries to the menu.
            this.menuEntries.Add(this.mapMenuEntry);
            this.menuEntries.Add(this.numberOfLivesMenuEntry);
            this.menuEntries.Add(this.gameTimeMenuEntry);
            this.menuEntries.Add(this.saveAndExitMenuEntry);

        } // End of the constructor

        #endregion

        #region Methods

        /// <summary>
        /// Fills in the latest values for the menu text.
        /// </summary>
        private void SetMenuEntryText()
        {
            // The current map
            if (this.maps.Count > 0)
                this.mapMenuEntry.Text = "Selected map: " + this.maps[currentMap];
            else
                this.mapMenuEntry.Text = "Selected map: 0 maps found";

            // The current game time
            this.gameTimeMenuEntry.Text = "Game time: " + this.currentGameTime + " minutes";

            // The current number of lives
            this.numberOfLivesMenuEntry.Text = "Number of lives: " + this.currentNumberOfLives;

        } // End of the SetMenuEntryText

        #endregion

        #region Event handlers

        /// <summary>
        /// Event handler for when the MapMenuEntry is selected.
        /// </summary>
        private void MapMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Calculate the current number.
            if (this.maps.Count > 0)
                this.currentMap = (this.currentMap + 1) % this.maps.Count;

            // Set the menu text.
            SetMenuEntryText();

        } // End of the MapMenuEntrySelected

        /// <summary>
        /// Event handler for when the NumberOfLivesMenuEntry is selected.
        /// </summary>
        private void NumberOfLivesMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Calculate the current number of lives.
            if (this.currentNumberOfLives >= 60)
            {
                this.currentNumberOfLives = 1;
            }
            else if (this.currentNumberOfLives < 10)
            {
                this.currentNumberOfLives++;
            }
            else if (this.currentNumberOfLives >= 10)
            {
                this.currentNumberOfLives += 5;
            }

            // Set the menu text.
            SetMenuEntryText();

        } // End of the NumberOfLivesMenuEntrySelected method

        /// <summary>
        /// Event handler for when the GameTimeMenuEntry is selected.
        /// </summary>
        private void GameTimeMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Calculate the current game time.
            if (this.currentGameTime >= 60)
            {
                this.currentGameTime = 1;
            }
            else if (this.currentGameTime < 10)
            {
                this.currentGameTime++;
            }
            else if (this.currentGameTime >= 10)
            {
                this.currentGameTime += 5;
            }

            // Set the menu text.
            SetMenuEntryText();

        } // End of the GameTimeMenuEntrySelected method

        /// <summary>
        /// Event handler for when the SaveAndExitMenuEntrySelected is selected.
        /// </summary>
        private void SaveAndExitMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Save the map number and the map name
            if (this.maps.Count > 0)
            {
                Program.selectedMap = this.currentMap;
                Program.selectedMapName = this.maps[this.currentMap];
            }

            // Set the number of lives in the program class
            Program.numberOfLives = this.currentNumberOfLives;

            // Set the game time in the program class.
            Program.totalGameTimeMilliseconds = this.currentGameTime * 60000;

            // Exit from this screen
            OnCancel(e.PlayerIndex);

        } // End of the SaveAndExitMenuEntrySelected method

        #endregion

    } // End of the class

} // End of the namespace
