#region File Description
//-----------------------------------------------------------------------------
// LobbyScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.Screens.MenuScreens;
using AvatarBlockWar.Screens.NetworkScreens;
using AvatarBlockWar.DataStorage;
using ScreenManagement;
using Networking;

namespace AvatarBlockWar.Screens.LobbyScreens
{
    /// <summary>
    /// The lobby screen provides a place for gamers to congregate before starting
    /// the actual gameplay. It displays a list of all the gamers in the session,
    /// and indicates which ones are currently talking. Each gamer can press a button
    /// to mark themselves as ready: gameplay will begin after everyone has done this.
    /// </summary>
    public class LobbyScreen : GameScreen
    {

        #region Variables

        // A reference to a network session
        public NetworkSession networkSession;

        // Talking textures
        Texture2D isReadyTexture;
        Texture2D hasVoiceTexture;
        Texture2D isTalkingTexture;
        Texture2D voiceMutedTexture;

        // Xbox icons
        Texture2D aButtonTexture;
        Texture2D bButtonTexture;
        Texture2D xButtonTexture;
        Texture2D yButtonTexture;

        // Pc icons
        Texture2D enterButtonTexture;
        Texture2D escapeButtonTexture;
        Texture2D xKeyButtonTexture;
        Texture2D zKeyButtonTexture;

        // Constant x and y offsets
        const int xOffset = 128;
        const int yOffset = 72;

        //// Variable used to syncronize settings for all network players.
        //bool hasUpdatedSettings = false;

        //// Variables for network communication
        //PacketWriter packetWriter = new PacketWriter();
        //PacketReader packetReader = new PacketReader();

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new lobby screen.
        /// </summary>
        public LobbyScreen(NetworkSession networkSession)
        {
            // Set values for instance variables
            this.networkSession = networkSession;
            this.TransitionOnTime = TimeSpan.FromSeconds(0.5);
            this.TransitionOffTime = TimeSpan.FromSeconds(0.5);

            // Set the selected map name
            string[] mapNames = MapDataStorage.GetMapNames(Program.commonStorageDevice);

            if(mapNames != null)
                Program.selectedMapName = mapNames[Program.selectedMap].Split('.')[0];

            // Test network latency
            // TODO: Uncomment or delete these rows.
            //this.networkSession.SimulatedLatency = TimeSpan.FromMilliseconds(1000);
            //this.networkSession.SimulatedPacketLoss = 0.1f;

        } // End of the constructor

        #endregion

        #region Initialization

        /// <summary>
        /// Loads graphics content used by the lobby screen.
        /// </summary>
        public override void LoadContent()
        {
            // Get a reference to a content manager
            ContentManager content = ScreenManager.Game.Content;

            // Load voice textures
            this.isReadyTexture = content.Load<Texture2D>("MenuTextures/chat_ready");
            this.hasVoiceTexture = content.Load<Texture2D>("MenuTextures/chat_able");
            this.isTalkingTexture = content.Load<Texture2D>("MenuTextures/chat_talking");
            this.voiceMutedTexture = content.Load<Texture2D>("MenuTextures/chat_mute");

            // Load button textures
            this.aButtonTexture = content.Load<Texture2D>("MenuTextures/aButton");
            this.bButtonTexture = content.Load<Texture2D>("MenuTextures/bButton");
            this.xButtonTexture = content.Load<Texture2D>("MenuTextures/xButton");
            this.yButtonTexture = content.Load<Texture2D>("MenuTextures/yButton");
            this.enterButtonTexture = content.Load<Texture2D>("MenuTextures/enterKey");
            this.escapeButtonTexture = content.Load<Texture2D>("MenuTextures/escapeKey");
            this.xKeyButtonTexture = content.Load<Texture2D>("MenuTextures/xKey");
            this.zKeyButtonTexture = content.Load<Texture2D>("MenuTextures/zKey");

        } // End of the LoadContent method.

        #endregion

        #region Update

        /// <summary>
        /// Update the lobby screen.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus, bool coveredByOtherScreen)
        {
            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

        } // End of the update method.

        #endregion

        #region Player input

        /// <summary>
        /// Handles user input for all the local gamers in the session. Unlike most
        /// screens, which use the InputState class to combine input data from all
        /// gamepads, the lobby needs to individually mark specific players as ready,
        /// so it loops over all the local gamers and reads their inputs individually.
        /// </summary>
        public override void HandleInput(InputState input)
        {
            // Handle input for each gamer in the network session.
            foreach (LocalNetworkGamer gamer in networkSession.LocalGamers)
            {
                PlayerIndex playerIndex = gamer.SignedInGamer.PlayerIndex;

                PlayerIndex unwantedOutput;

                if (input.IsMenuSelect(playerIndex, out unwantedOutput))
                {
                    HandleMenuSelect(gamer);
                }
                else if (input.IsMenuCancel(playerIndex, out unwantedOutput))
                {
                    HandleMenuCancel(gamer);
                }
                else if (input.IsSettingsMenuSelect(playerIndex, out unwantedOutput))
                {
                    HandleSettingsMenuSelect(gamer);
                }
            }

            // Handle input for all gamers that wants to join this game.
            PlayerIndex NotSignedInPlayerIndex;

            if (input.IsMenuJoinGame(null, out NotSignedInPlayerIndex))
            {
                // First, we need to make sure a suitable gamer profile is signed in.
                ProfileSignInScreen profileSignIn = new ProfileSignInScreen(NetworkSessionType.Local);

                // Hook up an event so once the ProfileSignInScreen is happy.
                profileSignIn.ProfileSignedIn += delegate
                {
                    // Handle the player join event.
                    HandleMenuJoinGame(Gamer.SignedInGamers[NotSignedInPlayerIndex]);

                };

                // Activate the ProfileSignInScreen.
                ScreenManager.AddScreen(profileSignIn, NotSignedInPlayerIndex);
            }

        } // End of the Handle input method

        #endregion

        #region Event handlers

        /// <summary>
        /// Handle MenuSelect inputs by marking ourselves as ready.
        /// </summary>
        private void HandleMenuSelect(LocalNetworkGamer gamer)
        {
            if (!gamer.IsReady)
            {
                gamer.IsReady = true;
            }
            else if (gamer.IsHost)
            {
                // The host has an option to force starting the game, even if not
                // everyone has marked themselves ready. If they press select twice
                // in a row, the first time marks the host ready, then the second
                // time we ask if they want to force start.
                MessageBoxScreen messageBox = new MessageBoxScreen(Resources.ConfirmForceStartGame);

                messageBox.Accepted += ConfirmStartGameMessageBoxAccepted;

                ScreenManager.AddScreen(messageBox, gamer.SignedInGamer.PlayerIndex);
            }

        } // End of the HandleMenuSelect method

        /// <summary>
        /// Handle MenuCancel inputs by clearing our ready status, or if it is
        /// already clear, prompting if the user wants to leave the session.
        /// </summary>
        private void HandleMenuCancel(LocalNetworkGamer gamer)
        {
            if (gamer.IsReady)
            {
                gamer.IsReady = false;
            }
            else
            {
                PlayerIndex playerIndex = gamer.SignedInGamer.PlayerIndex;
                NetworkSessionComponent.LeaveSession(ScreenManager, playerIndex);
            }

        } // End of the HandleMenuCancel method.

        /// <summary>
        /// Handle SettingsMenuSelect inputs
        /// </summary>
        private void HandleSettingsMenuSelect(LocalNetworkGamer gamer)
        {
            if (gamer.IsHost)
            {
                // Open the game settings menu
                ScreenManager.AddScreen(new GameSettingsScreen(), gamer.SignedInGamer.PlayerIndex);
            }

        } // End of the HandleSettingsMenuSelect method

        /// <summary>
        /// Handle MenuMenuJoinGame inputs.
        /// </summary>
        private void HandleMenuJoinGame(SignedInGamer gamer)
        {
            // Mark the playerExists flag as false.
            bool playerExists = false;

            // Loop throug each localNetworkGamer in the current network session.
            foreach (LocalNetworkGamer localGamer in networkSession.LocalGamers)
            {
                if (localGamer.Gamertag == gamer.Gamertag)
                {
                    playerExists = true;
                }
            }

            // Add the gamer if he does not exist.
            if (playerExists == false && networkSession.LocalGamers.Count < NetworkSessionComponent.MaxLocalGamers)
            {
                networkSession.AddLocalGamer(gamer);
            }
        }

        /// <summary>
        /// Event handler for when the host selects ok on the "are you sure you want to start?"
        /// </summary>
        private void ConfirmStartGameMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {
            if (networkSession.SessionState == NetworkSessionState.Lobby)
            {
                networkSession.StartGame();
            }

        } // End of the ConfirmStartGameMessageBoxAccepted method.

        #endregion

        #region Draw

        /// <summary>
        /// Draw the lobby screen.
        /// </summary>
        public override void Draw(GameTime gameTime)
        {

            SpriteBatch spriteBatch = ScreenManager.spriteBatch;
            //Viewport viewport = ScreenManager.GraphicsDevice.Viewport;
            SpriteFont font = ScreenManager.font;

            Vector2 position = new Vector2(xOffset, 150);

            // Make the lobby slide into place during transitions.
            float transitionOffset = (float)Math.Pow(TransitionPosition, 2);

            if (ScreenState == ScreenState.TransitionOn)
                position.X -= transitionOffset * 256;
            else
                position.X += transitionOffset * 512;

            spriteBatch.Begin();

            // Draw all the gamers in the session.
            int gamerCount = 0;

            foreach (NetworkGamer gamer in networkSession.AllGamers)
            {
                DrawGamer(gamer, position);

                // Advance to the next screen position, wrapping into two
                // columns if there are more than 8 gamers in the session.
                if (++gamerCount == 8)
                {
                    position.X += (1280 - (xOffset * 2)) / 2;
                    position.Y = 150;
                }
                else
                    position.Y += ScreenManager.font.LineSpacing;
            }

            // Draw the screen title.
            string title = Resources.Lobby;

            Vector2 titlePosition = new Vector2(xOffset, yOffset);
            Vector2 titleOrigin = Vector2.Zero;
            Color titleColor = Color.White * TransitionAlpha;
            float titleScale = 1.25f;

            titlePosition.Y -= transitionOffset * 100;

            spriteBatch.DrawString(font, title, new Vector2(titlePosition.X + 1, titlePosition.Y + 1), Color.DarkGray * TransitionAlpha, 0,
                                   titleOrigin, titleScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, title, titlePosition, titleColor, 0,
                                   titleOrigin, titleScale, SpriteEffects.None, 0);

            // Button panel
            float buttonTextScale = 1.25f;
            String selectText = "Ready";
            String backText = "Leave game/Not ready";
            String joinText = "Join game";
            String gameSettingsText = "Game settings";
            float selectStringSize = xOffset + 32 + 40 + (font.MeasureString(selectText).X * buttonTextScale) + 5;
            float backStringSize = selectStringSize + 32 + 40 + (font.MeasureString(backText).X * buttonTextScale) + 5;
            float joinStringSize = backStringSize + 32 + 40 + (font.MeasureString(joinText).X * buttonTextScale) + 5;

#if (!XBOX360)

            // Draw select button with text (enter button)
            spriteBatch.Draw(enterButtonTexture, new Vector2(xOffset, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, selectText, new Vector2(xOffset + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, selectText, new Vector2(xOffset + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);

            // Draw back button with text (esc button)
            spriteBatch.Draw(escapeButtonTexture, new Vector2(selectStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, backText, new Vector2(selectStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, backText, new Vector2(selectStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);

            // Draw join button with text (x button)
            spriteBatch.Draw(xKeyButtonTexture, new Vector2(backStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, joinText, new Vector2(backStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, joinText, new Vector2(backStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);

            if (networkSession.IsHost)
            {
                // Draw game settings button with text (z button)
                spriteBatch.Draw(zKeyButtonTexture, new Vector2(joinStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

                spriteBatch.DrawString(font, gameSettingsText, new Vector2(joinStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                       Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
                spriteBatch.DrawString(font, gameSettingsText, new Vector2(joinStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                       Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            }
#endif

#if (XBOX360)

            // Draw select key with text (a button)
            spriteBatch.Draw(aButtonTexture, new Vector2(xOffset, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, selectText, new Vector2(xOffset + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, selectText, new Vector2(xOffset + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);

            // Draw back button with text (b button)
            spriteBatch.Draw(bButtonTexture, new Vector2(selectStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, backText, new Vector2(selectStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, backText, new Vector2(selectStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);

            // Draw join button with text (x button)
            spriteBatch.Draw(xButtonTexture, new Vector2(backStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

            spriteBatch.DrawString(font, joinText, new Vector2(backStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(font, joinText, new Vector2(backStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                   Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            
            if (networkSession.IsHost)
            {
                // Draw game settings button with text (y button)
                spriteBatch.Draw(yButtonTexture, new Vector2(joinStringSize, 720 - yOffset - 42), Color.White * TransitionAlpha);

                spriteBatch.DrawString(font, gameSettingsText, new Vector2(joinStringSize + 41, 720 - yOffset - 40), Color.DarkGray * TransitionAlpha, 0,
                                       Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
                spriteBatch.DrawString(font, gameSettingsText, new Vector2(joinStringSize + 40, 720 - yOffset - 41), Color.White * TransitionAlpha, 0,
                                       Vector2.Zero, buttonTextScale, SpriteEffects.None, 0);
            }

#endif

            spriteBatch.End();
        }

        /// <summary>
        /// Helper draws the gamertag and status icons for a single NetworkGamer.
        /// </summary>
        private void DrawGamer(NetworkGamer gamer, Vector2 position)
        {
            SpriteBatch spriteBatch = ScreenManager.spriteBatch;
            SpriteFont font = ScreenManager.font;

            Vector2 iconWidth = new Vector2(34, 0);
            Vector2 iconOffset = new Vector2(0, 4);

            Vector2 iconPosition = position + iconOffset;

            // Draw the "is ready" icon.
            if (gamer.IsReady)
            {
                spriteBatch.Draw(isReadyTexture, iconPosition,
                                 Color.Lime * TransitionAlpha);
            }

            iconPosition += iconWidth;

            // Draw the "is muted", "is talking", or "has voice" icon.
            if (gamer.IsMutedByLocalUser)
            {
                spriteBatch.Draw(voiceMutedTexture, iconPosition,
                                 Color.Red * TransitionAlpha);
            }
            else if (gamer.IsTalking)
            {
                spriteBatch.Draw(isTalkingTexture, iconPosition,
                                 Color.Yellow * TransitionAlpha);
            }
            else if (gamer.HasVoice)
            {
                spriteBatch.Draw(hasVoiceTexture, iconPosition,
                                 Color.White * TransitionAlpha);
            }

            // Draw the gamertag, normally in white, but yellow for local players.
            string text = gamer.Gamertag;

            if (gamer.IsHost)
                text += Resources.HostSuffix;

            Color color = (gamer.IsLocal) ? Color.Yellow : Color.White;

            spriteBatch.DrawString(font, text, position + iconWidth * 2 + new Vector2(1,1), Color.DarkGray * TransitionAlpha);
            spriteBatch.DrawString(font, text, position + iconWidth * 2, color * TransitionAlpha);

        } // End of the DrawGamer method

        #endregion

    } // End of class

} // End of namespace
