﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AvatarBlockWar.DataStorage;
using ScreenManagement;

namespace AvatarBlockWar.Screens.MenuScreens
{
    /// <summary>
    /// This class represent a screen used to delete maps.
    /// </summary>
    public class DeleteMapScreen : MenuScreen
    {
        #region Variables

        // Menu entries
        private MenuEntry mapMenuEntry;
        private MenuEntry deleteMapMenuEntry;
        private MenuEntry exitMenuEntry;

        // Variables for map names
        private List<string> maps;
        private int currentMap = 0;

        #endregion

        #region Constructors

        /// <summary>
        /// Create a new delete map screen
        /// </summary>
        public DeleteMapScreen()
            : base(Resources.DeleteMap)
        {
            // Get all map names
            string[] mapNames = MapDataStorage.GetMapNames(Program.commonStorageDevice);

            // Create the list of map names
            if (mapNames != null)
            {
                this.maps = new List<string>(mapNames.Length);
                for (int map = 0; map < mapNames.Length; map++)
                {
                    // Add the map name, without the extension, to the list of map names.
                    this.maps.Add(mapNames[map].Split('.')[0]);
                }
            }

            // Create menu entries
            this.mapMenuEntry = new MenuEntry("");
            this.deleteMapMenuEntry = new MenuEntry("Delete the selected map");
            this.exitMenuEntry = new MenuEntry("Exit");

            // Set the mapMenuEntry text.
            SetMenuEntryText();

            // Hook up menu event handlers
            this.mapMenuEntry.Selected += MapMenuEntrySelected;
            this.deleteMapMenuEntry.Selected += DeleteMapMenuEntrySelected;
            this.exitMenuEntry.Selected += ExitMenuEntrySelected;

            // Add menu entries to the menu.
            this.menuEntries.Add(this.mapMenuEntry);
            this.menuEntries.Add(this.deleteMapMenuEntry);
            this.menuEntries.Add(this.exitMenuEntry);

        } // End of the constructor

        #endregion

        #region Methods

        /// <summary>
        /// Fills in the latest values for the menu text.
        /// </summary>
        private void SetMenuEntryText()
        {
            if (this.maps != null && this.maps.Count > 0)
                this.mapMenuEntry.Text = "Selected map: " + this.maps[currentMap];
            else
                this.mapMenuEntry.Text = "Selected map: 0 maps found";

        } // End of the SetMenuEntryText

        #endregion

        #region Event handlers

        /// <summary>
        /// Event handler for when the MapMenuEntry is selected.
        /// </summary>
        private void MapMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Calculate the current number.
            if(this.maps.Count > 0)
                this.currentMap = (this.currentMap + 1) % this.maps.Count;

            // Set the menu text.
            SetMenuEntryText();

        } // End of the MapMenuEntrySelected

        /// <summary>
        /// Event handler for when the DeleteMapMenuEntry is selected.
        /// </summary>
        private void DeleteMapMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            if (this.maps.Count > 0)
            {
                // Create a confirmation message box
                MessageBoxScreen confirmDeleteMessageBox = new MessageBoxScreen(Resources.ConfirmDeleteMap);

                // Add a method to handle the ok choice
                confirmDeleteMessageBox.Accepted += ConfirmDeleteMessageBoxAccepted;

                ScreenManager.AddScreen(confirmDeleteMessageBox, e.PlayerIndex);
 
            }

        } // End of the DeleteMapMenuEntrySelected method

        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure you want to delete" message box.
        /// </summary>
        private void ConfirmDeleteMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {
            // Delete the selected map
            MapDataStorage.DeleteMap(Program.commonStorageDevice, this.maps[this.currentMap]);

            // Remove the map name from the list
            this.maps.RemoveAt(this.currentMap);

            // Calculate the current number.
            if (this.maps.Count > 0)
                this.currentMap = (this.currentMap + 1) % this.maps.Count;

            // Set the menu text.
            SetMenuEntryText();

        } // End of the ConfirmDeleteMessageBoxAccepted method

        /// <summary>
        /// Event handler for when the ExitMenuEntry is selected.
        /// </summary>
        private void ExitMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Exit from this screen
            OnCancel(e.PlayerIndex);

        } // End of the ExitMenuEntrySelected method

        #endregion

    } // End of the class

} // End of the namespace
