#region File Description
//-----------------------------------------------------------------------------
// MainMenuScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using AvatarBlockWar.Screens.NetworkScreens;
using AvatarBlockWar.Screens.LobbyScreens;
using AvatarBlockWar.DataStorage;
using ScreenManagement;
using Networking;

namespace AvatarBlockWar.Screens.MenuScreens
{
    /// <summary>
    /// The main menu screen is the first thing displayed when the game starts up.
    /// </summary>
    public class MainMenuScreen : MenuScreen
    {
        
        #region Instance Variables

        // A menu entry to buy the game.
        MenuEntry buyMenuEntry;
        bool isTrialGame = true;
        
        #endregion

        #region Constructors

        /// <summary>
        /// Create a new main menu screen.
        /// </summary>
        public MainMenuScreen()
            : base(Resources.MainMenu)
        {
            // Create our menu entries.
            MenuEntry buildMapMenuEntry = new MenuEntry(Resources.BuildMap);
            MenuEntry localGameMenuEntry = new MenuEntry(Resources.LocalMultiplayer);
            MenuEntry systemLinkMenuEntry = new MenuEntry(Resources.SystemLink);
            MenuEntry liveMenuEntry = new MenuEntry(Resources.PlayerMatch);
            MenuEntry optionsMenuEntry = new MenuEntry(Resources.Options);
            buyMenuEntry = new MenuEntry("Buy this game");
            MenuEntry exitMenuEntry = new MenuEntry(Resources.Exit);

            // Hook up menu event handlers.
            buildMapMenuEntry.Selected += BuildMapMenuEntrySelected;
            localGameMenuEntry.Selected += LocalGameMenuEntrySelected;
            systemLinkMenuEntry.Selected += SystemLinkMenuEntrySelected; 
            liveMenuEntry.Selected += LiveMenuEntrySelected;
            optionsMenuEntry.Selected += OptionsMenuEntrySelected;
            buyMenuEntry.Selected += ShowMarketPlace;
            exitMenuEntry.Selected += OnCancel;

            // Add entries to the menu.
            menuEntries.Add(buildMapMenuEntry);
            menuEntries.Add(localGameMenuEntry);
            menuEntries.Add(systemLinkMenuEntry);
            menuEntries.Add(liveMenuEntry);
            menuEntries.Add(optionsMenuEntry);
            menuEntries.Add(buyMenuEntry);
            menuEntries.Add(exitMenuEntry);
        
        } // End of the constructor

        #endregion

        #region Update

        /// <summary>
        /// Update the main menu screen.
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="otherScreenHasFocus"></param>
        /// <param name="coveredByOtherScreen"></param>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus, bool coveredByOtherScreen)
        {
            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

            // Show the buy menu entry if the game is a trial game.
            if (this.isTrialGame == true)
            {
                if (Guide.IsTrialMode == false)
                {
                    menuEntries.Remove(buyMenuEntry);
                    this.isTrialGame = false;
                }
            }
        } // End of the Update method.
        
        #endregion

        #region Event Handlers

        /// <summary>
        /// Event handler for when the build map menu entry is selected.
        /// </summary>
        private void BuildMapMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            
            // First, we need to make sure a suitable gamer profile is signed in.
            ProfileSignInScreen profileSignIn = new ProfileSignInScreen(NetworkSessionType.Local);

            // Hook up an event so once the ProfileSignInScreen is happy
            profileSignIn.ProfileSignedIn += delegate
            {
                // Get a storage device for the player profile.
                MapDataStorage.SelectCommonStorageDevice();

                // Add the build map screen
                ScreenManager.AddScreen(new BuildMapScreen(), e.PlayerIndex);

            };

            // Activate the ProfileSignInScreen.
            ScreenManager.AddScreen(profileSignIn, e.PlayerIndex);
    
        } // End of the BuildMapMenuEntrySelected method

        /// <summary>
        /// Event handler for when the local game menu entry is selected.
        /// </summary>
        private void LocalGameMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // First, we need to make sure a suitable gamer profile is signed in.
            ProfileSignInScreen profileSignIn = new ProfileSignInScreen(NetworkSessionType.Local);

            // Hook up an event so once the ProfileSignInScreen is happy,
            // it will activate the StartSplitScreenSessionOperation.
            profileSignIn.ProfileSignedIn += delegate
            {
                // Get a storage device for the player profile.
                MapDataStorage.SelectCommonStorageDevice();

                // Call the method to start the operation for a split screen game.
                StartLocalGameSessionOperation(e.PlayerIndex);

            };

            // Activate the ProfileSignInScreen.
            ScreenManager.AddScreen(profileSignIn, e.PlayerIndex);

        } // End of the LocalGameMenuEntrySelected method

        /// <summary>
        /// Begin to start a local game.
        /// </summary>
        /// <param name="playerIndex"></param>
        private void StartLocalGameSessionOperation(PlayerIndex playerIndex)
        {

            //try
            //{
                // Which local profiles should we include in this session?
                List<SignedInGamer> localGamers = new List<SignedInGamer>();
                localGamers.Add(Gamer.SignedInGamers[playerIndex]);

                // Begin an asynchronous create network session operation.
                IAsyncResult asyncResult = NetworkSession.BeginCreate(
                                                    NetworkSessionType.Local, localGamers,
                                                    NetworkSessionComponent.MaxGamers,
                                                    0, null, null, null);

                // Activate the network busy screen, which will display an animation until this operation has completed.
                NetworkBusyScreen busyScreen = new NetworkBusyScreen(asyncResult);

                busyScreen.OperationCompleted += LocalGameSessionOperationCompleted;

                ScreenManager.AddScreen(busyScreen, playerIndex);
            //}
            //catch (Exception exception)
            //{
            //    NetworkErrorScreen errorScreen = new NetworkErrorScreen(exception);

            //    ScreenManager.AddScreen(errorScreen, playerIndex);
            //}

        } // End of the StartLocalGameSessionOperation method

        /// <summary>
        /// Event handler for when the asynchronous create network session operation has completed.
        /// </summary>
        private void LocalGameSessionOperationCompleted(object sender, OperationCompletedEventArgs e)
        {
            //try
            //{
                // End the asynchronous create network session operation.
                NetworkSession networkSession = NetworkSession.EndCreate(e.AsyncResult);

                // Create a component that will manage the session we just created.
                NetworkSessionComponent.Create(ScreenManager, networkSession);

                // Go to the lobby screen. We pass null as the controlling player,
                // because the lobby screen accepts input from all local players
                // who are in the session, not just a single controlling player.
                ScreenManager.AddScreen(new LocalGameLobbyScreen(networkSession), null);
            //}
            //catch (Exception exception)
            //{
            //    NetworkErrorScreen errorScreen = new NetworkErrorScreen(exception);

            //    ScreenManager.AddScreen(errorScreen, ControllingPlayer);
            //}

        } // End of the LocalGameSessionOperationCompleted method.

        /// <summary>
        /// Event handler for when the Live menu entry is selected.
        /// </summary>
        private void LiveMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            CreateOrFindSession(NetworkSessionType.PlayerMatch, e.PlayerIndex);
        }

        /// <summary>
        /// Event handler for when the System Link menu entry is selected.
        /// </summary>
        private void SystemLinkMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            CreateOrFindSession(NetworkSessionType.SystemLink, e.PlayerIndex);
        }

        /// <summary>
        /// Event handler for when the Options menu entry is selected.
        /// </summary>
        private void OptionsMenuEntrySelected(object sender, PlayerIndexEventArgs e)
        {
            // Add the options screen.
            ScreenManager.AddScreen(new OptionScreen(), e.PlayerIndex);
        }

        /// <summary>
        /// Helper method shared by the Live and System Link menu event handlers.
        /// </summary>
        private void CreateOrFindSession(NetworkSessionType sessionType,
                                 PlayerIndex playerIndex)
        {
            // First, we need to make sure a suitable gamer profile is signed in.
            ProfileSignInScreen profileSignIn = new ProfileSignInScreen(sessionType);

            // Hook up an event so once the ProfileSignInScreen is happy,
            // it will activate the CreateOrFindSessionScreen.
            profileSignIn.ProfileSignedIn += delegate
            {
                GameScreen createOrFind = new CreateOrFindSessionScreen(sessionType);

                ScreenManager.AddScreen(createOrFind, playerIndex);
            };

            // Activate the ProfileSignInScreen.
            ScreenManager.AddScreen(profileSignIn, playerIndex);
        }

        /// <summary>
        /// When the user cancels the main menu, ask if they want to exit the sample.
        /// </summary>
        protected override void OnCancel(PlayerIndex playerIndex)
        {
            MessageBoxScreen confirmExitMessageBox =
                                    new MessageBoxScreen(Resources.ConfirmExitSample);

            confirmExitMessageBox.Accepted += ConfirmExitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmExitMessageBox, playerIndex);

        } // End of the OnCancel method

        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure
        /// you want to exit" message box.
        /// </summary>
        private void ConfirmExitMessageBoxAccepted(object sender, PlayerIndexEventArgs e)
        {
            ScreenManager.Game.Exit();

        } // End of the ConfirmExitMessageBoxAccepted

        /// <summary>
        /// Event handler to show the marketplace.
        /// </summary>
        private void ShowMarketPlace(object sender, PlayerIndexEventArgs e)
        {
            // Create a message box screens.
            MessageBoxScreen confirmSignIn = new MessageBoxScreen(Resources.ConfirmSignIn);

            // Show the market place if the gamer are signed in with an online gamer profile.
            if (ProfileSignInScreen.CanPlayerBuyGame(e.PlayerIndex))
            {
                Guide.ShowMarketplace(e.PlayerIndex);
            }
            else
            {
                confirmSignIn.Accepted += delegate
                {
                    Guide.ShowSignIn(1, true);
                };

                ScreenManager.AddScreen(confirmSignIn, e.PlayerIndex);  
            }

        } // End of the ShowMarketPlace method

        #endregion

    } // End of the class

} // End of namespace
