// Camera settings.
float4x4 ViewProjection;

// Lightning
float3 LightDirection = normalize(float3(-1, -1, -1));
float3 DiffuseLight = 1.25;
float3 AmbientLight = 0.25;

// Texture map
texture cubeTextureMap;

// Texture sampler
sampler cubeTextureSampler = sampler_state { Texture = (cubeTextureMap); AddressU = CLAMP; AddressV = CLAMP; };

// Input struct for the VertexShader, VertexPositionNormalTexture vertex
struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float2 TextureCoordinate : TEXCOORD0;
};

// Input struct for cube instances
struct VertexInstanceInput
{
    float4x4 world : BLENDWEIGHT;
	float2 textureOffset : TEXCOORD1;
};

// Output structure for the VertexShader, is input for the PixelShader
struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
	float2 textureOffset : TEXCOORD1;
};

// Vertex shader function // float4x4 world : BLENDWEIGHT, float texNumber : TEXCOORD1
VertexShaderOutput VertexShaderInstancing(VertexShaderInput input, VertexInstanceInput instanceInput)
{
    VertexShaderOutput output;

    // Apply the world and camera matrices to compute the output position.
	output.Position = mul(mul(input.Position, transpose(instanceInput.world)), ViewProjection);

    // Compute lighting, using a simple Lambert model.
    float3 worldNormal = mul(input.Normal, instanceInput.world);
    
    float diffuseAmount = max(-dot(worldNormal, LightDirection), 0);
    
    float3 lightingResult = saturate(diffuseAmount * DiffuseLight + AmbientLight);
    
    output.Color = float4(lightingResult, 1);

    // Copy across the input texture coordinate.
    output.TextureCoordinate = input.TextureCoordinate;

	// Set the cube texture offset
	output.textureOffset = instanceInput.textureOffset;

    return output;
}

// This pixel shader returns the color to draw
float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{

	return tex2D(cubeTextureSampler, input.TextureCoordinate + input.textureOffset) * input.Color;
}

// Hardware instancing technique.
technique HardwareInstancing
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 VertexShaderInstancing();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
